# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2021-2022 Valory AG
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""This module contains the tests of the ledger API connection for the contract APIs."""
import asyncio
from typing import Any, Dict, List, Tuple, cast
from unittest import mock

import pytest
from aea.common import Address
from aea.contracts.base import Contract
from aea.crypto.registries import make_crypto
from aea.helpers.transaction.base import RawMessage, RawTransaction, State
from aea.mail.base import Envelope
from aea.protocols.base import Message
from aea.protocols.dialogue.base import Dialogue
from aea_ledger_ethereum import EthereumCrypto

from packages.valory.connections.ledger.connection import LedgerConnection
from packages.valory.contracts.gnosis_safe.contract import (
    PUBLIC_ID as GNOSIS_SAFE_PUBLIC_ID,
)
from packages.valory.protocols.contract_api.dialogues import ContractApiDialogue
from packages.valory.protocols.contract_api.dialogues import (
    ContractApiDialogues as BaseContractApiDialogues,
)
from packages.valory.protocols.contract_api.message import ContractApiMessage

from tests.conftest import ETHEREUM_KEY_DEPLOYER, get_key


SOME_SKILL_ID = "some/skill:0.1.0"


class ContractApiDialogues(BaseContractApiDialogues):
    """This class keeps track of all contract_api dialogues."""

    def __init__(self, self_address: str) -> None:
        """
        Initialize dialogues.

        :param self_address: the address of the entity for whom dialogues are maintained
        """

        def role_from_first_message(  # pylint: disable=unused-argument
            message: Message, receiver_address: Address
        ) -> Dialogue.Role:
            """Infer the role of the agent from an incoming/outgoing first message.

            :param message: an incoming/outgoing first message
            :param receiver_address: the address of the receiving agent
            :return: The role of the agent
            """
            return ContractApiDialogue.Role.AGENT

        BaseContractApiDialogues.__init__(
            self,
            self_address=self_address,
            role_from_first_message=role_from_first_message,
        )


@pytest.mark.usefixtures("gnosis_safe_hardhat_scope_class")
class TestContractDispatcher:
    """Test contract dispatcher."""

    @pytest.mark.asyncio
    async def test_get_deploy_transaction(
        self,
        gnosis_safe_contract: Tuple[Contract, str],
        ledger_apis_connection: LedgerConnection,
        owners: List[str],
        threshold: int,
    ) -> None:
        """
        Test get deploy transaction with contract gnosis_safe_contract.

        :param gnosis_safe_contract: fixture
        :param ledger_apis_connection: fixture
        :param owners: fixture
        :param threshold: fixture
        """
        _, contract_address = gnosis_safe_contract
        contract_api_dialogues = ContractApiDialogues(SOME_SKILL_ID)
        crypto = make_crypto(
            EthereumCrypto.identifier, private_key_path=ETHEREUM_KEY_DEPLOYER
        )
        request, contract_api_dialogue = contract_api_dialogues.create(
            counterparty=str(ledger_apis_connection.connection_id),
            performative=ContractApiMessage.Performative.GET_DEPLOY_TRANSACTION,
            ledger_id=EthereumCrypto.identifier,
            contract_id=str(GNOSIS_SAFE_PUBLIC_ID),
            callable="get_deploy_transaction",
            kwargs=ContractApiMessage.Kwargs(
                body=dict(
                    deployer_address=crypto.address,
                    gas=5000000,
                    gas_price=10 * 10,
                    owners=owners,
                    threshold=threshold,
                )
            ),
        )
        envelope = Envelope(
            to=request.to,
            sender=request.sender,
            message=request,
        )

        await ledger_apis_connection.send(envelope)
        await asyncio.sleep(0.01)
        response = await ledger_apis_connection.receive()

        assert response is not None
        assert isinstance(response.message, ContractApiMessage)
        response_message = cast(ContractApiMessage, response.message)
        assert (
            response_message.performative
            == ContractApiMessage.Performative.RAW_TRANSACTION
        ), "Error: {}".format(response_message.message)
        response_dialogue = contract_api_dialogues.update(response_message)
        assert response_dialogue == contract_api_dialogue
        assert isinstance(response_message.raw_transaction, RawTransaction)
        assert response_message.raw_transaction.ledger_id == EthereumCrypto.identifier
        assert len(response.message.raw_transaction.body) == 9
        assert len(str(response.message.raw_transaction.body["data"])) > 0
        assert (
            response.message.raw_transaction.body["contract_address"]
            != contract_address
        ), "new deployment should have new address"

    @pytest.mark.asyncio
    async def test_get_deploy_transaction_with_validate_and_call_callable(
        self,
        gnosis_safe_contract: Tuple[Contract, str],
        ledger_apis_connection: LedgerConnection,
        owners: List[str],
        threshold: int,
    ) -> None:
        """
        Test get deploy transaction with contract gnosis_safe_contract ( using _validate_and_call_callable instead of _stub_call method ).

        :param gnosis_safe_contract: fixture
        :param ledger_apis_connection: fixture
        :param owners: fixture
        :param threshold: fixture
        """
        _, contract_address = gnosis_safe_contract
        contract_api_dialogues = ContractApiDialogues(SOME_SKILL_ID)
        crypto = make_crypto(
            EthereumCrypto.identifier, private_key_path=ETHEREUM_KEY_DEPLOYER
        )
        request, contract_api_dialogue = contract_api_dialogues.create(
            counterparty=str(ledger_apis_connection.connection_id),
            performative=ContractApiMessage.Performative.GET_DEPLOY_TRANSACTION,
            ledger_id=EthereumCrypto.identifier,
            contract_id=str(GNOSIS_SAFE_PUBLIC_ID),
            callable="get_deploy_transaction",
            kwargs=ContractApiMessage.Kwargs(
                body=dict(
                    deployer_address=crypto.address,
                    gas=5000000,
                    gas_price=10 ** 10,
                    owners=owners,
                    threshold=threshold,
                )
            ),
        )
        envelope = Envelope(
            to=request.to,
            sender=request.sender,
            message=request,
        )

        with mock.patch.object(
            ledger_apis_connection._contract_dispatcher, "_call_stub", return_value=None
        ):
            await ledger_apis_connection.send(envelope)
            await asyncio.sleep(0.01)
            response = await ledger_apis_connection.receive()

        assert response is not None
        assert isinstance(response.message, ContractApiMessage)
        response_message = cast(ContractApiMessage, response.message)
        assert (
            response_message.performative
            == ContractApiMessage.Performative.RAW_TRANSACTION
        ), "Error: {}".format(response_message.message)
        response_dialogue = contract_api_dialogues.update(response_message)
        assert response_dialogue == contract_api_dialogue
        assert isinstance(response_message.raw_transaction, RawTransaction)
        assert response_message.raw_transaction.ledger_id == EthereumCrypto.identifier
        assert len(response.message.raw_transaction.body) == 9
        assert len(str(response.message.raw_transaction.body["data"])) > 0
        assert (
            response.message.raw_transaction.body["contract_address"]
            != contract_address
        ), "new deployment should have new address"

    @pytest.mark.asyncio
    async def test_get_state(
        self,
        gnosis_safe_contract: Tuple[Contract, str],
        ledger_apis_connection: LedgerConnection,
    ) -> None:
        """
        Test get state with contract gnosis_safe_contract.

        :param gnosis_safe_contract: fixture
        :param ledger_apis_connection: fixture
        """
        _, contract_address = gnosis_safe_contract
        contract_api_dialogues = ContractApiDialogues(SOME_SKILL_ID)
        request, contract_api_dialogue = contract_api_dialogues.create(
            counterparty=str(ledger_apis_connection.connection_id),
            performative=ContractApiMessage.Performative.GET_STATE,
            ledger_id=EthereumCrypto.identifier,
            contract_id=str(GNOSIS_SAFE_PUBLIC_ID),
            callable="get_state",
            contract_address=contract_address,
            kwargs=ContractApiMessage.Kwargs(
                {"agent_address": get_key(ETHEREUM_KEY_DEPLOYER), "token_id": 1}
            ),
        )
        envelope = Envelope(
            to=request.to,
            sender=request.sender,
            message=request,
        )

        with mock.patch(
            "packages.valory.contracts.gnosis_safe.contract.GnosisSafeContract.get_state",
            return_value={},
        ):
            await ledger_apis_connection.send(envelope)
            await asyncio.sleep(0.01)
            response = await ledger_apis_connection.receive()

        assert response is not None
        assert isinstance(response.message, ContractApiMessage)
        response_message = cast(ContractApiMessage, response.message)
        assert (
            response_message.performative == ContractApiMessage.Performative.STATE
        ), "Error: {}".format(response_message.message)
        response_dialogue = contract_api_dialogues.update(response_message)
        assert response_dialogue == contract_api_dialogue
        assert isinstance(response_message.state, State)
        assert response_message.state.ledger_id == EthereumCrypto.identifier
        assert len(response.message.state.body) == 0

    @pytest.mark.asyncio
    async def test_get_state_with_validate_and_call_callable(
        self,
        gnosis_safe_contract: Tuple[Contract, str],
        ledger_apis_connection: LedgerConnection,
    ) -> None:
        """
        Test get state with contract gnosis_safe_contract ( using _validate_and_call_callable instead of _call_stub method).

        :param gnosis_safe_contract: fixture
        :param ledger_apis_connection: fixture
        """
        _, contract_address = gnosis_safe_contract
        contract_api_dialogues = ContractApiDialogues(SOME_SKILL_ID)
        request, contract_api_dialogue = contract_api_dialogues.create(
            counterparty=str(ledger_apis_connection.connection_id),
            performative=ContractApiMessage.Performative.GET_STATE,
            ledger_id=EthereumCrypto.identifier,
            contract_id=str(GNOSIS_SAFE_PUBLIC_ID),
            callable="get_state",
            contract_address=contract_address,
            kwargs=ContractApiMessage.Kwargs(
                {"agent_address": get_key(ETHEREUM_KEY_DEPLOYER), "token_id": 1}
            ),
        )
        envelope = Envelope(
            to=request.to,
            sender=request.sender,
            message=request,
        )

        with mock.patch.object(
            ledger_apis_connection._contract_dispatcher, "_call_stub", return_value=None
        ):

            def get_state(
                ledger_api: Any, contract_address: str, *args: Any, **kwargs: Any
            ) -> Dict:
                """Mock `get_state` method from GnosisSafeContract."""
                return {}

            with mock.patch(
                "packages.valory.contracts.gnosis_safe.contract.GnosisSafeContract.get_state",
                new_callable=lambda: get_state,
            ):
                await ledger_apis_connection.send(envelope)
                await asyncio.sleep(0.01)
                response = await ledger_apis_connection.receive()

        assert response is not None
        assert isinstance(response.message, ContractApiMessage)
        response_message = cast(ContractApiMessage, response.message)
        assert (
            response_message.performative == ContractApiMessage.Performative.STATE
        ), "Error: {}".format(response_message.message)
        response_dialogue = contract_api_dialogues.update(response_message)
        assert response_dialogue == contract_api_dialogue
        assert isinstance(response_message.state, State)
        assert response_message.state.ledger_id == EthereumCrypto.identifier
        assert len(response.message.state.body) == 0

    @pytest.mark.asyncio
    async def test_get_raw_transaction(
        self,
        gnosis_safe_contract: Tuple[Contract, str],
        ledger_apis_connection: LedgerConnection,
    ) -> None:
        """
        Test get raw transaction with contract get_raw_transaction.

        NOTE: we already deploy it once at base!

        :param gnosis_safe_contract: fixture
        :param ledger_apis_connection: fixture
        """
        _, contract_address = gnosis_safe_contract
        contract_api_dialogues = ContractApiDialogues(SOME_SKILL_ID)
        request, contract_api_dialogue = contract_api_dialogues.create(
            counterparty=str(ledger_apis_connection.connection_id),
            performative=ContractApiMessage.Performative.GET_RAW_TRANSACTION,
            ledger_id=EthereumCrypto.identifier,
            contract_id=str(GNOSIS_SAFE_PUBLIC_ID),
            callable="get_raw_transaction",
            contract_address=contract_address,
            kwargs=ContractApiMessage.Kwargs(
                {"agent_address": get_key(ETHEREUM_KEY_DEPLOYER), "token_id": 1}
            ),
        )
        envelope = Envelope(
            to=request.to,
            sender=request.sender,
            message=request,
        )

        with mock.patch(
            "packages.valory.contracts.gnosis_safe.contract.GnosisSafeContract.get_raw_transaction",
            return_value={},
        ):
            await ledger_apis_connection.send(envelope)
            await asyncio.sleep(0.01)
            response = await ledger_apis_connection.receive()

        assert response is not None
        assert isinstance(response.message, ContractApiMessage)
        response_message = cast(ContractApiMessage, response.message)
        assert (
            response_message.performative
            == ContractApiMessage.Performative.RAW_TRANSACTION
        ), "Error: {}".format(response_message.message)
        response_dialogue = contract_api_dialogues.update(response_message)
        assert response_dialogue == contract_api_dialogue
        assert isinstance(response_message.raw_transaction, RawTransaction)
        assert response_message.raw_transaction.ledger_id == EthereumCrypto.identifier
        assert len(response.message.raw_transaction.body) == 0

    @pytest.mark.asyncio
    async def test_get_raw_message(
        self,
        gnosis_safe_contract: Tuple[Contract, str],
        ledger_apis_connection: LedgerConnection,
    ) -> None:
        """
        Test get raw message with contract get_raw_transaction.

        :param gnosis_safe_contract: fixture
        :param ledger_apis_connection: fixture
        """
        _, contract_address = gnosis_safe_contract
        contract_api_dialogues = ContractApiDialogues(SOME_SKILL_ID)
        request, contract_api_dialogue = contract_api_dialogues.create(
            counterparty=str(ledger_apis_connection.connection_id),
            performative=ContractApiMessage.Performative.GET_RAW_MESSAGE,
            ledger_id=EthereumCrypto.identifier,
            contract_id=str(GNOSIS_SAFE_PUBLIC_ID),
            callable="get_raw_message",
            contract_address=contract_address,
            kwargs=ContractApiMessage.Kwargs(
                {"agent_address": get_key(ETHEREUM_KEY_DEPLOYER), "token_id": 1}
            ),
        )
        envelope = Envelope(
            to=request.to,
            sender=request.sender,
            message=request,
        )

        with mock.patch(
            "packages.valory.contracts.gnosis_safe.contract.GnosisSafeContract.get_raw_message",
            return_value=b"{}",
        ):
            await ledger_apis_connection.send(envelope)
            await asyncio.sleep(0.01)
            response = await ledger_apis_connection.receive()

        assert response is not None
        assert isinstance(response.message, ContractApiMessage)
        response_message = cast(ContractApiMessage, response.message)
        assert (
            response_message.performative == ContractApiMessage.Performative.RAW_MESSAGE
        ), "Error: {}".format(response_message.message)
        response_dialogue = contract_api_dialogues.update(response_message)
        assert response_dialogue == contract_api_dialogue
        assert isinstance(response_message.raw_message, RawMessage)
        assert response_message.raw_message.ledger_id == EthereumCrypto.identifier
        assert response.message.raw_message.body == b"{}"

    @pytest.mark.asyncio
    async def test_get_error_message(
        self,
        gnosis_safe_contract: Tuple[Contract, str],
        ledger_apis_connection: LedgerConnection,
        owners: List[str],
        threshold: int,
    ) -> None:
        """
        Test get_error_message method of contract dispatcher.

        :param gnosis_safe_contract: fixture
        :param ledger_apis_connection: fixture
        :param owners: fixture
        :param threshold: fixture
        """
        _, contract_address = gnosis_safe_contract
        contract_api_dialogues = ContractApiDialogues(SOME_SKILL_ID)
        crypto = make_crypto(
            EthereumCrypto.identifier, private_key_path=ETHEREUM_KEY_DEPLOYER
        )
        request, contract_api_dialogue = contract_api_dialogues.create(
            counterparty=str(ledger_apis_connection.connection_id),
            performative=ContractApiMessage.Performative.GET_DEPLOY_TRANSACTION,
            ledger_id=EthereumCrypto.identifier,
            contract_id=str(GNOSIS_SAFE_PUBLIC_ID),
            callable="callable",
            kwargs=ContractApiMessage.Kwargs(
                body=dict(
                    deployer_address=crypto.address,
                    gas=5000000,
                    owners=owners,
                    threshold=threshold,
                )
            ),
        )
        envelope = Envelope(
            to=request.to,
            sender=request.sender,
            message=request,
        )

        with mock.patch.object(
            ledger_apis_connection._contract_dispatcher.contract_registry,  # type: ignore
            "make",
            return_value=None,
        ):
            await ledger_apis_connection.send(envelope)
            await asyncio.sleep(0.01)
            response = await ledger_apis_connection.receive()

        assert response is not None
        assert isinstance(response.message, ContractApiMessage)
        response_message = cast(ContractApiMessage, response.message)
        assert (
            response_message.performative == ContractApiMessage.Performative.ERROR
        ), "Error: {}".format(response_message.message)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tendermint/crypto/keys.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database


# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from packages.valory.connections.abci.gogoproto import (
    gogo_pb2 as gogoproto_dot_gogo__pb2,
)


DESCRIPTOR = _descriptor.FileDescriptor(
    name="tendermint/crypto/keys.proto",
    package="tendermint.crypto",
    syntax="proto3",
    serialized_options=b"Z8github.com/tendermint/tendermint/proto/tendermint/crypto",
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n\x1ctendermint/crypto/keys.proto\x12\x11tendermint.crypto\x1a\x14gogoproto/gogo.proto"D\n\tPublicKey\x12\x11\n\x07\x65\x64\x32\x35\x35\x31\x39\x18\x01 \x01(\x0cH\x00\x12\x13\n\tsecp256k1\x18\x02 \x01(\x0cH\x00:\x08\xe8\xa1\x1f\x01\xe8\xa0\x1f\x01\x42\x05\n\x03sumB:Z8github.com/tendermint/tendermint/proto/tendermint/cryptob\x06proto3',
    dependencies=[
        gogoproto_dot_gogo__pb2.DESCRIPTOR,
    ],
)


_PUBLICKEY = _descriptor.Descriptor(
    name="PublicKey",
    full_name="tendermint.crypto.PublicKey",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="ed25519",
            full_name="tendermint.crypto.PublicKey.ed25519",
            index=0,
            number=1,
            type=12,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"",
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="secp256k1",
            full_name="tendermint.crypto.PublicKey.secp256k1",
            index=1,
            number=2,
            type=12,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"",
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=b"\350\241\037\001\350\240\037\001",
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[
        _descriptor.OneofDescriptor(
            name="sum",
            full_name="tendermint.crypto.PublicKey.sum",
            index=0,
            containing_type=None,
            create_key=_descriptor._internal_create_key,
            fields=[],
        ),
    ],
    serialized_start=73,
    serialized_end=141,
)

_PUBLICKEY.oneofs_by_name["sum"].fields.append(_PUBLICKEY.fields_by_name["ed25519"])
_PUBLICKEY.fields_by_name["ed25519"].containing_oneof = _PUBLICKEY.oneofs_by_name["sum"]
_PUBLICKEY.oneofs_by_name["sum"].fields.append(_PUBLICKEY.fields_by_name["secp256k1"])
_PUBLICKEY.fields_by_name["secp256k1"].containing_oneof = _PUBLICKEY.oneofs_by_name[
    "sum"
]
DESCRIPTOR.message_types_by_name["PublicKey"] = _PUBLICKEY
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

PublicKey = _reflection.GeneratedProtocolMessageType(
    "PublicKey",
    (_message.Message,),
    {
        "DESCRIPTOR": _PUBLICKEY,
        "__module__": "tendermint.crypto.keys_pb2"
        # @@protoc_insertion_point(class_scope:tendermint.crypto.PublicKey)
    },
)
_sym_db.RegisterMessage(PublicKey)


DESCRIPTOR._options = None
_PUBLICKEY._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2021-2022 Valory AG
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Test the abci_app_chain.py module of the skill."""

import pytest

from packages.valory.skills.abstract_round_abci.abci_app_chain import (
    AbciAppTransitionMapping,
    chain,
)
from packages.valory.skills.abstract_round_abci.base import AbciApp, DegenerateRound

from tests.helpers.base import make_round_class


class TestAbciAppChaining:
    """Test chaning of AbciApps."""

    def setup(self) -> None:
        """Setup test."""
        self.round_1a = make_round_class("round_1a")
        self.round_1b = make_round_class("round_1b")
        self.round_1c = make_round_class("round_1c", (DegenerateRound,))

        self.round_2a = make_round_class("round_2a")
        self.round_2b = make_round_class("round_2b")
        self.round_2c = make_round_class("round_2c", (DegenerateRound,))
        self.round_2d = make_round_class("round_2d")

        self.round_3a = make_round_class("round_3a")
        self.round_3b = make_round_class("round_3b")
        self.round_3c = make_round_class("round_3c", (DegenerateRound,))

        self.event_1a = "event_1a"
        self.event_1b = "event_1b"
        self.event_1c = "event_1c"
        self.event_timeout1 = "timeout_1"

        self.event_2a = "event_2a"
        self.event_2b = "event_2b"
        self.event_2c = "event_2c"
        self.event_timeout2 = "timeout_2"

        self.event_3a = "event_3a"
        self.event_3b = "event_3b"
        self.event_3c = "event_3c"
        self.event_timeout3 = "timeout_3"

        self.timeout1 = 10.0
        self.timeout2 = 15.0
        self.timeout3 = 20.0

        class AbciApp1(AbciApp):
            initial_round_cls = self.round_1a
            transition_function = {
                self.round_1a: {
                    self.event_timeout1: self.round_1a,
                    self.event_1b: self.round_1b,
                },
                self.round_1b: {
                    self.event_1a: self.round_1a,
                    self.event_1c: self.round_1c,
                },
                self.round_1c: {},
            }
            final_states = {self.round_1c}
            event_to_timeout = {self.event_timeout1: self.timeout1}

        self.app1_class = AbciApp1

        class AbciApp2(AbciApp):
            initial_round_cls = self.round_2a
            transition_function = {
                self.round_2a: {
                    self.event_timeout2: self.round_2a,
                    self.event_2b: self.round_2b,
                },
                self.round_2b: {
                    self.event_2a: self.round_2a,
                    self.event_2c: self.round_2c,
                },
                self.round_2c: {},
            }
            final_states = {self.round_2c}
            event_to_timeout = {self.event_timeout2: self.timeout2}

        self.app2_class = AbciApp2

        class AbciApp3(AbciApp):
            initial_round_cls = self.round_3a
            transition_function = {
                self.round_3a: {
                    self.event_timeout3: self.round_3a,
                    self.event_3b: self.round_3b,
                },
                self.round_3b: {
                    self.event_3a: self.round_3a,
                    self.event_3c: self.round_3c,
                },
                self.round_3c: {},
            }
            final_states = {self.round_3c}
            event_to_timeout = {self.event_timeout3: self.timeout3}

        self.app3_class = AbciApp3

        class AbciApp2Faulty1(AbciApp):
            initial_round_cls = self.round_2a
            transition_function = {
                self.round_2a: {
                    self.event_timeout2: self.round_2a,
                    self.event_2b: self.round_2b,
                },
                self.round_2b: {
                    self.event_2a: self.round_2a,
                    self.event_2c: self.round_2c,
                },
                self.round_2c: {},
            }
            final_states = {self.round_2c}
            event_to_timeout = {self.event_timeout1: self.timeout2}

        self.app2_class_faulty1 = AbciApp2Faulty1

    def test_chain_two(self) -> None:
        """Test the AbciApp chain function."""

        abci_app_transition_mapping: AbciAppTransitionMapping = {
            self.round_1c: self.round_2a,
            self.round_2c: self.round_1a,
        }

        ComposedAbciApp = chain((self.app1_class, self.app2_class), abci_app_transition_mapping)  # type: ignore

        assert ComposedAbciApp.initial_round_cls == self.round_1a
        assert ComposedAbciApp.transition_function == {
            self.round_1a: {
                self.event_timeout1: self.round_1a,
                self.event_1b: self.round_1b,
            },
            self.round_1b: {
                self.event_1a: self.round_1a,
                self.event_1c: self.round_2a,
            },
            self.round_2a: {
                self.event_timeout2: self.round_2a,
                self.event_2b: self.round_2b,
            },
            self.round_2b: {
                self.event_2a: self.round_2a,
                self.event_2c: self.round_1a,
            },
        }
        assert ComposedAbciApp.final_states == set()
        assert ComposedAbciApp.event_to_timeout == {
            self.event_timeout1: self.timeout1,
            self.event_timeout2: self.timeout2,
        }

    def test_chain_three(self) -> None:
        """Test the AbciApp chain function."""

        abci_app_transition_mapping: AbciAppTransitionMapping = {
            self.round_1c: self.round_2a,
            self.round_2c: self.round_3a,
        }

        ComposedAbciApp = chain((self.app1_class, self.app2_class, self.app3_class), abci_app_transition_mapping)  # type: ignore

        assert ComposedAbciApp.initial_round_cls == self.round_1a
        assert ComposedAbciApp.transition_function == {
            self.round_1a: {
                self.event_timeout1: self.round_1a,
                self.event_1b: self.round_1b,
            },
            self.round_1b: {
                self.event_1a: self.round_1a,
                self.event_1c: self.round_2a,
            },
            self.round_2a: {
                self.event_timeout2: self.round_2a,
                self.event_2b: self.round_2b,
            },
            self.round_2b: {
                self.event_2a: self.round_2a,
                self.event_2c: self.round_3a,
            },
            self.round_3a: {
                self.event_timeout3: self.round_3a,
                self.event_3b: self.round_3b,
            },
            self.round_3b: {
                self.event_3a: self.round_3a,
                self.event_3c: self.round_3c,
            },
            self.round_3c: {},
        }
        assert ComposedAbciApp.final_states == {self.round_3c}
        assert ComposedAbciApp.event_to_timeout == {
            self.event_timeout1: self.timeout1,
            self.event_timeout2: self.timeout2,
            self.event_timeout3: self.timeout3,
        }

    def test_chain_two_negative_timeouts(self) -> None:
        """Test the AbciApp chain function."""

        abci_app_transition_mapping: AbciAppTransitionMapping = {
            self.round_1c: self.round_2a,
            self.round_2c: self.round_1a,
        }

        with pytest.raises(
            ValueError, match="but it is already defined in a prior app with timeout"
        ):
            _ = chain((self.app1_class, self.app2_class_faulty1), abci_app_transition_mapping)  # type: ignore

    def test_chain_two_negative_mapping_initial_states(self) -> None:
        """Test the AbciApp chain function."""

        abci_app_transition_mapping: AbciAppTransitionMapping = {
            self.round_1c: self.round_2b,
            self.round_2c: self.round_1a,
        }

        with pytest.raises(ValueError, match="Found non-initial state"):
            _ = chain((self.app1_class, self.app2_class), abci_app_transition_mapping)  # type: ignore

    def test_chain_two_negative_mapping_final_states(self) -> None:
        """Test the AbciApp chain function."""

        abci_app_transition_mapping: AbciAppTransitionMapping = {
            self.round_1c: self.round_2a,
            self.round_2b: self.round_1a,
        }

        with pytest.raises(ValueError, match="Found non-final state"):
            _ = chain((self.app1_class, self.app2_class), abci_app_transition_mapping)  # type: ignore



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2021-2022 Valory AG
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""Encode AEA messages into Tendermint protobuf messages."""
# pylint: disable=no-member
from typing import Callable, Optional, Union, cast

from packages.valory.connections.abci.tendermint.abci.types_pb2 import (  # type: ignore
    ConsensusParams,
    Event,
    Request,
    Response,
    ResponseApplySnapshotChunk,
    ResponseBeginBlock,
    ResponseCheckTx,
    ResponseCommit,
    ResponseDeliverTx,
    ResponseEcho,
    ResponseEndBlock,
    ResponseException,
    ResponseFlush,
    ResponseInfo,
    ResponseInitChain,
    ResponseListSnapshots,
    ResponseLoadSnapshotChunk,
    ResponseOfferSnapshot,
    ResponseQuery,
    ResponseSetOption,
    Snapshot,
    ValidatorUpdate,
)
from packages.valory.connections.abci.tendermint.crypto.proof_pb2 import (  # type: ignore
    ProofOps,
)
from packages.valory.protocols.abci import AbciMessage
from packages.valory.protocols.abci.custom_types import (
    ConsensusParams as CustomConsensusParams,  # type: ignore
)
from packages.valory.protocols.abci.custom_types import Event as CustomEvent
from packages.valory.protocols.abci.custom_types import ProofOps as CustomProofOps
from packages.valory.protocols.abci.custom_types import Snapshot as CustomSnapshot
from packages.valory.protocols.abci.custom_types import (
    ValidatorUpdate as CustomValidatorUpdate,
)


class _TendermintProtocolEncoder:
    """
    Decoder called by the server to process requests *towards* the TCP connection with Tendermint.

    It translates from the AEA's ABCI protocol messages into Tendermint's ABCI Protobuf messages.
    """

    @classmethod
    def process(cls, message: AbciMessage) -> Optional[Union[Request, Response]]:
        """Encode an AbciMessage object into either Request or Respose protobuf objects."""
        # from "(request|response)_type", get 'type'
        handler: Callable[[AbciMessage], Union[Request, Response]] = getattr(
            cls, message.performative.value, cls.no_match
        )
        return handler(message)

    @classmethod
    def response_exception(cls, message: AbciMessage) -> Response:
        """
        Process the response exception.

        :param message: the response.
        :return: the ABCI protobuf object.
        """
        exception = ResponseException()
        exception.error = message.error
        response = Response(exception=exception)
        return response

    @classmethod
    def response_echo(cls, message: AbciMessage) -> Response:
        """
        Process the response echo.

        :param message: the response.
        :return: the ABCI protobuf object.
        """
        echo = ResponseEcho()
        echo.message = message.message
        response = Response(echo=echo)
        return response

    @classmethod
    def response_set_option(cls, message: AbciMessage) -> Response:
        """
        Process the response set_option.

        :param message: the response.
        :return: the ABCI protobuf object.
        """
        set_option = ResponseSetOption()
        set_option.code = message.code
        set_option.log = message.log
        set_option.info = message.info
        response = Response(set_option=set_option)
        return response

    @classmethod
    def response_info(cls, message: AbciMessage) -> Response:
        """
        Process the response info.

        :param message: the response.
        :return: the ABCI protobuf object.
        """
        info = ResponseInfo()
        info.data = message.info_data
        info.version = message.version
        info.app_version = message.app_version
        info.last_block_height = message.last_block_height
        info.last_block_app_hash = message.last_block_app_hash
        response = Response(info=info)
        return response

    @classmethod
    def response_flush(cls, _message: AbciMessage) -> Response:
        """
        Process the response flush.

        :param _message: the response.
        :return: the ABCI protobuf object.
        """
        response = Response(flush=ResponseFlush())
        return response

    @classmethod
    def response_init_chain(cls, message: AbciMessage) -> Response:
        """
        Process the response init_chain.

        :param message: the response.
        :return: the ABCI protobuf object.
        """
        init_chain = ResponseInitChain()

        if message.consensus_params is not None:
            consensus_params_pb = cls._encode_consensus_params(message.consensus_params)
            init_chain.consensus_params.CopyFrom(consensus_params_pb)

        validators_pb = [
            cls._encode_validator_update(vu)
            for vu in list(message.validators.validator_updates)
        ]
        init_chain.validators.extend(validators_pb)

        init_chain.app_hash = message.app_hash

        response = Response(init_chain=init_chain)
        return response

    @classmethod
    def response_query(cls, message: AbciMessage) -> Response:
        """
        Process the response query.

        :param message: the response.
        :return: the ABCI protobuf object.
        """
        query = ResponseQuery()

        query.code = message.code
        query.log = message.log
        query.info = message.info
        query.index = message.index
        query.key = message.key
        query.value = message.value

        proof_ops_pb = cls._encode_proof_ops(message.proof_ops)
        query.proof_ops.CopyFrom(proof_ops_pb)

        query.height = message.height
        query.codespace = message.codespace

        response = Response(query=query)
        return response

    @classmethod
    def response_check_tx(cls, message: AbciMessage) -> Response:
        """
        Process the response check_tx.

        :param message: the response.
        :return: the ABCI protobuf object.
        """
        check_tx = ResponseCheckTx()

        check_tx.code = message.code
        check_tx.data = message.data
        check_tx.log = message.log
        check_tx.info = message.info
        check_tx.gas_wanted = message.gas_wanted
        check_tx.gas_used = message.gas_used

        events_pb = [cls._encode_event(e) for e in message.events.events]
        check_tx.events.extend(events_pb)

        check_tx.codespace = message.codespace

        response = Response(check_tx=check_tx)
        return response

    @classmethod
    def response_deliver_tx(cls, message: AbciMessage) -> Response:
        """
        Process the response deliver_tx.

        :param message: the response.
        :return: the ABCI protobuf object.
        """
        deliver_tx = ResponseDeliverTx()

        deliver_tx.code = message.code
        deliver_tx.data = message.data
        deliver_tx.log = message.log
        deliver_tx.info = message.info
        deliver_tx.gas_wanted = message.gas_wanted
        deliver_tx.gas_used = message.gas_used

        events_pb = [cls._encode_event(e) for e in message.events.events]
        deliver_tx.events.extend(events_pb)

        deliver_tx.codespace = message.codespace

        response = Response(deliver_tx=deliver_tx)
        return response

    @classmethod
    def response_begin_block(cls, message: AbciMessage) -> Response:
        """
        Process the response begin_block.

        :param message: the response.
        :return: the ABCI protobuf object.
        """
        begin_block = ResponseBeginBlock()

        events_pb = [cls._encode_event(e) for e in message.events.events]
        begin_block.events.extend(events_pb)

        response = Response(begin_block=begin_block)
        return response

    @classmethod
    def response_end_block(cls, message: AbciMessage) -> Response:
        """
        Process the response end_block.

        :param message: the response.
        :return: the ABCI protobuf object.
        """
        end_block = ResponseEndBlock()

        if (
            message.is_set("consensus_param_updates")
            and message.consensus_param_updates is not None
        ):
            consensus_params_updates = cast(
                ConsensusParams, message.consensus_param_updates
            )
            consensus_params_pb = cls._encode_consensus_params(consensus_params_updates)
            end_block.consensus_param_updates.CopyFrom(consensus_params_pb)

        validator_updates_pb = [
            cls._encode_validator_update(vu)
            for vu in message.validator_updates.validator_updates
        ]
        end_block.validator_updates.extend(validator_updates_pb)

        events_pb = [cls._encode_event(e) for e in message.events.events]
        end_block.events.extend(events_pb)

        response = Response(end_block=end_block)
        return response

    @classmethod
    def response_commit(cls, message: AbciMessage) -> Response:
        """
        Process the response commit.

        :param message: the response.
        :return: the ABCI protobuf object.
        """
        commit = ResponseCommit()
        commit.data = message.data
        commit.retain_height = message.retain_height
        return Response(commit=commit)

    @classmethod
    def response_list_snapshots(cls, message: AbciMessage) -> Response:
        """
        Process the response list_snapshots.

        :param message: the response.
        :return: the ABCI protobuf object.
        """
        list_snapshots = ResponseListSnapshots()
        snapshots_pb = [
            cls._encode_snapshot(snapshot) for snapshot in message.snapshots.snapshots
        ]
        list_snapshots.snapshots.extend(snapshots_pb)
        return Response(list_snapshots=list_snapshots)

    @classmethod
    def response_offer_snapshot(cls, message: AbciMessage) -> Response:
        """
        Process the response offer_snapshot.

        :param message: the response.
        :return: the ABCI protobuf object.
        """
        offer_snapshot = ResponseOfferSnapshot()
        offer_snapshot.result = message.result.result_type.value
        return Response(offer_snapshot=offer_snapshot)

    @classmethod
    def response_load_snapshot_chunk(cls, message: AbciMessage) -> Response:
        """
        Process the response load_snapshot_chunk.

        :param message: the response.
        :return: the ABCI protobuf object.
        """
        load_snapshot_chunk = ResponseLoadSnapshotChunk()
        load_snapshot_chunk.chunk = message.chunk
        return Response(load_snapshot_chunk=load_snapshot_chunk)

    @classmethod
    def response_apply_snapshot_chunk(cls, message: AbciMessage) -> Response:
        """
        Process the response apply_snapshot_chunk.

        :param message: the response.
        :return: the ABCI protobuf object.
        """
        apply_snapshot_chunk = ResponseApplySnapshotChunk()
        apply_snapshot_chunk.result = message.result.result_type.value
        apply_snapshot_chunk.refetch_chunks.extend(message.refetch_chunks)
        apply_snapshot_chunk.reject_senders.extend(message.reject_senders)
        return Response(apply_snapshot_chunk=apply_snapshot_chunk)

    @classmethod
    def no_match(cls, _request: Request) -> None:  # pragma: nocover
        """No match."""
        return None

    @classmethod
    def _encode_consensus_params(
        cls, consensus_params: CustomConsensusParams
    ) -> ConsensusParams:
        consensus_params_pb = ConsensusParams()
        CustomConsensusParams.encode(consensus_params_pb, consensus_params)
        return consensus_params_pb

    @classmethod
    def _encode_validator_update(
        cls, validator_update: CustomValidatorUpdate
    ) -> ValidatorUpdate:
        validator_update_pb = ValidatorUpdate()
        CustomValidatorUpdate.encode(validator_update_pb, validator_update)
        return validator_update_pb

    @classmethod
    def _encode_event(cls, event: CustomEvent) -> Event:
        event_pb = Event()
        CustomEvent.encode(event_pb, event)
        return event_pb

    @classmethod
    def _encode_proof_ops(cls, proof_ops: CustomProofOps) -> ProofOps:
        proof_ops_pb = ProofOps()
        CustomProofOps.encode(proof_ops_pb, proof_ops)
        return proof_ops_pb

    @classmethod
    def _encode_snapshot(cls, snapshot: CustomSnapshot) -> Event:
        snapshot_pb = Snapshot()
        CustomSnapshot.encode(snapshot_pb, snapshot)
        return snapshot_pb



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: gogoproto/gogo.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database


# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import descriptor_pb2 as google_dot_protobuf_dot_descriptor__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
    name="gogoproto/gogo.proto",
    package="gogoproto",
    syntax="proto2",
    serialized_options=b'\n\023com.google.protobufB\nGoGoProtosZ"github.com/gogo/protobuf/gogoproto',
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n\x14gogoproto/gogo.proto\x12\tgogoproto\x1a google/protobuf/descriptor.proto:;\n\x13goproto_enum_prefix\x12\x1c.google.protobuf.EnumOptions\x18\xb1\xe4\x03 \x01(\x08:=\n\x15goproto_enum_stringer\x12\x1c.google.protobuf.EnumOptions\x18\xc5\xe4\x03 \x01(\x08:5\n\renum_stringer\x12\x1c.google.protobuf.EnumOptions\x18\xc6\xe4\x03 \x01(\x08:7\n\x0f\x65num_customname\x12\x1c.google.protobuf.EnumOptions\x18\xc7\xe4\x03 \x01(\t:0\n\x08\x65numdecl\x12\x1c.google.protobuf.EnumOptions\x18\xc8\xe4\x03 \x01(\x08:A\n\x14\x65numvalue_customname\x12!.google.protobuf.EnumValueOptions\x18\xd1\x83\x04 \x01(\t:;\n\x13goproto_getters_all\x12\x1c.google.protobuf.FileOptions\x18\x99\xec\x03 \x01(\x08:?\n\x17goproto_enum_prefix_all\x12\x1c.google.protobuf.FileOptions\x18\x9a\xec\x03 \x01(\x08:<\n\x14goproto_stringer_all\x12\x1c.google.protobuf.FileOptions\x18\x9b\xec\x03 \x01(\x08:9\n\x11verbose_equal_all\x12\x1c.google.protobuf.FileOptions\x18\x9c\xec\x03 \x01(\x08:0\n\x08\x66\x61\x63\x65_all\x12\x1c.google.protobuf.FileOptions\x18\x9d\xec\x03 \x01(\x08:4\n\x0cgostring_all\x12\x1c.google.protobuf.FileOptions\x18\x9e\xec\x03 \x01(\x08:4\n\x0cpopulate_all\x12\x1c.google.protobuf.FileOptions\x18\x9f\xec\x03 \x01(\x08:4\n\x0cstringer_all\x12\x1c.google.protobuf.FileOptions\x18\xa0\xec\x03 \x01(\x08:3\n\x0bonlyone_all\x12\x1c.google.protobuf.FileOptions\x18\xa1\xec\x03 \x01(\x08:1\n\tequal_all\x12\x1c.google.protobuf.FileOptions\x18\xa5\xec\x03 \x01(\x08:7\n\x0f\x64\x65scription_all\x12\x1c.google.protobuf.FileOptions\x18\xa6\xec\x03 \x01(\x08:3\n\x0btestgen_all\x12\x1c.google.protobuf.FileOptions\x18\xa7\xec\x03 \x01(\x08:4\n\x0c\x62\x65nchgen_all\x12\x1c.google.protobuf.FileOptions\x18\xa8\xec\x03 \x01(\x08:5\n\rmarshaler_all\x12\x1c.google.protobuf.FileOptions\x18\xa9\xec\x03 \x01(\x08:7\n\x0funmarshaler_all\x12\x1c.google.protobuf.FileOptions\x18\xaa\xec\x03 \x01(\x08:<\n\x14stable_marshaler_all\x12\x1c.google.protobuf.FileOptions\x18\xab\xec\x03 \x01(\x08:1\n\tsizer_all\x12\x1c.google.protobuf.FileOptions\x18\xac\xec\x03 \x01(\x08:A\n\x19goproto_enum_stringer_all\x12\x1c.google.protobuf.FileOptions\x18\xad\xec\x03 \x01(\x08:9\n\x11\x65num_stringer_all\x12\x1c.google.protobuf.FileOptions\x18\xae\xec\x03 \x01(\x08:<\n\x14unsafe_marshaler_all\x12\x1c.google.protobuf.FileOptions\x18\xaf\xec\x03 \x01(\x08:>\n\x16unsafe_unmarshaler_all\x12\x1c.google.protobuf.FileOptions\x18\xb0\xec\x03 \x01(\x08:B\n\x1agoproto_extensions_map_all\x12\x1c.google.protobuf.FileOptions\x18\xb1\xec\x03 \x01(\x08:@\n\x18goproto_unrecognized_all\x12\x1c.google.protobuf.FileOptions\x18\xb2\xec\x03 \x01(\x08:8\n\x10gogoproto_import\x12\x1c.google.protobuf.FileOptions\x18\xb3\xec\x03 \x01(\x08:6\n\x0eprotosizer_all\x12\x1c.google.protobuf.FileOptions\x18\xb4\xec\x03 \x01(\x08:3\n\x0b\x63ompare_all\x12\x1c.google.protobuf.FileOptions\x18\xb5\xec\x03 \x01(\x08:4\n\x0ctypedecl_all\x12\x1c.google.protobuf.FileOptions\x18\xb6\xec\x03 \x01(\x08:4\n\x0c\x65numdecl_all\x12\x1c.google.protobuf.FileOptions\x18\xb7\xec\x03 \x01(\x08:<\n\x14goproto_registration\x12\x1c.google.protobuf.FileOptions\x18\xb8\xec\x03 \x01(\x08:7\n\x0fmessagename_all\x12\x1c.google.protobuf.FileOptions\x18\xb9\xec\x03 \x01(\x08:=\n\x15goproto_sizecache_all\x12\x1c.google.protobuf.FileOptions\x18\xba\xec\x03 \x01(\x08:;\n\x13goproto_unkeyed_all\x12\x1c.google.protobuf.FileOptions\x18\xbb\xec\x03 \x01(\x08::\n\x0fgoproto_getters\x12\x1f.google.protobuf.MessageOptions\x18\x81\xf4\x03 \x01(\x08:;\n\x10goproto_stringer\x12\x1f.google.protobuf.MessageOptions\x18\x83\xf4\x03 \x01(\x08:8\n\rverbose_equal\x12\x1f.google.protobuf.MessageOptions\x18\x84\xf4\x03 \x01(\x08:/\n\x04\x66\x61\x63\x65\x12\x1f.google.protobuf.MessageOptions\x18\x85\xf4\x03 \x01(\x08:3\n\x08gostring\x12\x1f.google.protobuf.MessageOptions\x18\x86\xf4\x03 \x01(\x08:3\n\x08populate\x12\x1f.google.protobuf.MessageOptions\x18\x87\xf4\x03 \x01(\x08:3\n\x08stringer\x12\x1f.google.protobuf.MessageOptions\x18\xc0\x8b\x04 \x01(\x08:2\n\x07onlyone\x12\x1f.google.protobuf.MessageOptions\x18\x89\xf4\x03 \x01(\x08:0\n\x05\x65qual\x12\x1f.google.protobuf.MessageOptions\x18\x8d\xf4\x03 \x01(\x08:6\n\x0b\x64\x65scription\x12\x1f.google.protobuf.MessageOptions\x18\x8e\xf4\x03 \x01(\x08:2\n\x07testgen\x12\x1f.google.protobuf.MessageOptions\x18\x8f\xf4\x03 \x01(\x08:3\n\x08\x62\x65nchgen\x12\x1f.google.protobuf.MessageOptions\x18\x90\xf4\x03 \x01(\x08:4\n\tmarshaler\x12\x1f.google.protobuf.MessageOptions\x18\x91\xf4\x03 \x01(\x08:6\n\x0bunmarshaler\x12\x1f.google.protobuf.MessageOptions\x18\x92\xf4\x03 \x01(\x08:;\n\x10stable_marshaler\x12\x1f.google.protobuf.MessageOptions\x18\x93\xf4\x03 \x01(\x08:0\n\x05sizer\x12\x1f.google.protobuf.MessageOptions\x18\x94\xf4\x03 \x01(\x08:;\n\x10unsafe_marshaler\x12\x1f.google.protobuf.MessageOptions\x18\x97\xf4\x03 \x01(\x08:=\n\x12unsafe_unmarshaler\x12\x1f.google.protobuf.MessageOptions\x18\x98\xf4\x03 \x01(\x08:A\n\x16goproto_extensions_map\x12\x1f.google.protobuf.MessageOptions\x18\x99\xf4\x03 \x01(\x08:?\n\x14goproto_unrecognized\x12\x1f.google.protobuf.MessageOptions\x18\x9a\xf4\x03 \x01(\x08:5\n\nprotosizer\x12\x1f.google.protobuf.MessageOptions\x18\x9c\xf4\x03 \x01(\x08:2\n\x07\x63ompare\x12\x1f.google.protobuf.MessageOptions\x18\x9d\xf4\x03 \x01(\x08:3\n\x08typedecl\x12\x1f.google.protobuf.MessageOptions\x18\x9e\xf4\x03 \x01(\x08:6\n\x0bmessagename\x12\x1f.google.protobuf.MessageOptions\x18\xa1\xf4\x03 \x01(\x08:<\n\x11goproto_sizecache\x12\x1f.google.protobuf.MessageOptions\x18\xa2\xf4\x03 \x01(\x08::\n\x0fgoproto_unkeyed\x12\x1f.google.protobuf.MessageOptions\x18\xa3\xf4\x03 \x01(\x08:1\n\x08nullable\x12\x1d.google.protobuf.FieldOptions\x18\xe9\xfb\x03 \x01(\x08:.\n\x05\x65mbed\x12\x1d.google.protobuf.FieldOptions\x18\xea\xfb\x03 \x01(\x08:3\n\ncustomtype\x12\x1d.google.protobuf.FieldOptions\x18\xeb\xfb\x03 \x01(\t:3\n\ncustomname\x12\x1d.google.protobuf.FieldOptions\x18\xec\xfb\x03 \x01(\t:0\n\x07jsontag\x12\x1d.google.protobuf.FieldOptions\x18\xed\xfb\x03 \x01(\t:1\n\x08moretags\x12\x1d.google.protobuf.FieldOptions\x18\xee\xfb\x03 \x01(\t:1\n\x08\x63\x61sttype\x12\x1d.google.protobuf.FieldOptions\x18\xef\xfb\x03 \x01(\t:0\n\x07\x63\x61stkey\x12\x1d.google.protobuf.FieldOptions\x18\xf0\xfb\x03 \x01(\t:2\n\tcastvalue\x12\x1d.google.protobuf.FieldOptions\x18\xf1\xfb\x03 \x01(\t:0\n\x07stdtime\x12\x1d.google.protobuf.FieldOptions\x18\xf2\xfb\x03 \x01(\x08:4\n\x0bstdduration\x12\x1d.google.protobuf.FieldOptions\x18\xf3\xfb\x03 \x01(\x08:3\n\nwktpointer\x12\x1d.google.protobuf.FieldOptions\x18\xf4\xfb\x03 \x01(\x08\x42\x45\n\x13\x63om.google.protobufB\nGoGoProtosZ"github.com/gogo/protobuf/gogoproto',
    dependencies=[
        google_dot_protobuf_dot_descriptor__pb2.DESCRIPTOR,
    ],
)


GOPROTO_ENUM_PREFIX_FIELD_NUMBER = 62001
goproto_enum_prefix = _descriptor.FieldDescriptor(
    name="goproto_enum_prefix",
    full_name="gogoproto.goproto_enum_prefix",
    index=0,
    number=62001,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
GOPROTO_ENUM_STRINGER_FIELD_NUMBER = 62021
goproto_enum_stringer = _descriptor.FieldDescriptor(
    name="goproto_enum_stringer",
    full_name="gogoproto.goproto_enum_stringer",
    index=1,
    number=62021,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
ENUM_STRINGER_FIELD_NUMBER = 62022
enum_stringer = _descriptor.FieldDescriptor(
    name="enum_stringer",
    full_name="gogoproto.enum_stringer",
    index=2,
    number=62022,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
ENUM_CUSTOMNAME_FIELD_NUMBER = 62023
enum_customname = _descriptor.FieldDescriptor(
    name="enum_customname",
    full_name="gogoproto.enum_customname",
    index=3,
    number=62023,
    type=9,
    cpp_type=9,
    label=1,
    has_default_value=False,
    default_value=b"".decode("utf-8"),
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
ENUMDECL_FIELD_NUMBER = 62024
enumdecl = _descriptor.FieldDescriptor(
    name="enumdecl",
    full_name="gogoproto.enumdecl",
    index=4,
    number=62024,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
ENUMVALUE_CUSTOMNAME_FIELD_NUMBER = 66001
enumvalue_customname = _descriptor.FieldDescriptor(
    name="enumvalue_customname",
    full_name="gogoproto.enumvalue_customname",
    index=5,
    number=66001,
    type=9,
    cpp_type=9,
    label=1,
    has_default_value=False,
    default_value=b"".decode("utf-8"),
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
GOPROTO_GETTERS_ALL_FIELD_NUMBER = 63001
goproto_getters_all = _descriptor.FieldDescriptor(
    name="goproto_getters_all",
    full_name="gogoproto.goproto_getters_all",
    index=6,
    number=63001,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
GOPROTO_ENUM_PREFIX_ALL_FIELD_NUMBER = 63002
goproto_enum_prefix_all = _descriptor.FieldDescriptor(
    name="goproto_enum_prefix_all",
    full_name="gogoproto.goproto_enum_prefix_all",
    index=7,
    number=63002,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
GOPROTO_STRINGER_ALL_FIELD_NUMBER = 63003
goproto_stringer_all = _descriptor.FieldDescriptor(
    name="goproto_stringer_all",
    full_name="gogoproto.goproto_stringer_all",
    index=8,
    number=63003,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
VERBOSE_EQUAL_ALL_FIELD_NUMBER = 63004
verbose_equal_all = _descriptor.FieldDescriptor(
    name="verbose_equal_all",
    full_name="gogoproto.verbose_equal_all",
    index=9,
    number=63004,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
FACE_ALL_FIELD_NUMBER = 63005
face_all = _descriptor.FieldDescriptor(
    name="face_all",
    full_name="gogoproto.face_all",
    index=10,
    number=63005,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
GOSTRING_ALL_FIELD_NUMBER = 63006
gostring_all = _descriptor.FieldDescriptor(
    name="gostring_all",
    full_name="gogoproto.gostring_all",
    index=11,
    number=63006,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
POPULATE_ALL_FIELD_NUMBER = 63007
populate_all = _descriptor.FieldDescriptor(
    name="populate_all",
    full_name="gogoproto.populate_all",
    index=12,
    number=63007,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
STRINGER_ALL_FIELD_NUMBER = 63008
stringer_all = _descriptor.FieldDescriptor(
    name="stringer_all",
    full_name="gogoproto.stringer_all",
    index=13,
    number=63008,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
ONLYONE_ALL_FIELD_NUMBER = 63009
onlyone_all = _descriptor.FieldDescriptor(
    name="onlyone_all",
    full_name="gogoproto.onlyone_all",
    index=14,
    number=63009,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
EQUAL_ALL_FIELD_NUMBER = 63013
equal_all = _descriptor.FieldDescriptor(
    name="equal_all",
    full_name="gogoproto.equal_all",
    index=15,
    number=63013,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
DESCRIPTION_ALL_FIELD_NUMBER = 63014
description_all = _descriptor.FieldDescriptor(
    name="description_all",
    full_name="gogoproto.description_all",
    index=16,
    number=63014,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
TESTGEN_ALL_FIELD_NUMBER = 63015
testgen_all = _descriptor.FieldDescriptor(
    name="testgen_all",
    full_name="gogoproto.testgen_all",
    index=17,
    number=63015,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
BENCHGEN_ALL_FIELD_NUMBER = 63016
benchgen_all = _descriptor.FieldDescriptor(
    name="benchgen_all",
    full_name="gogoproto.benchgen_all",
    index=18,
    number=63016,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
MARSHALER_ALL_FIELD_NUMBER = 63017
marshaler_all = _descriptor.FieldDescriptor(
    name="marshaler_all",
    full_name="gogoproto.marshaler_all",
    index=19,
    number=63017,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
UNMARSHALER_ALL_FIELD_NUMBER = 63018
unmarshaler_all = _descriptor.FieldDescriptor(
    name="unmarshaler_all",
    full_name="gogoproto.unmarshaler_all",
    index=20,
    number=63018,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
STABLE_MARSHALER_ALL_FIELD_NUMBER = 63019
stable_marshaler_all = _descriptor.FieldDescriptor(
    name="stable_marshaler_all",
    full_name="gogoproto.stable_marshaler_all",
    index=21,
    number=63019,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
SIZER_ALL_FIELD_NUMBER = 63020
sizer_all = _descriptor.FieldDescriptor(
    name="sizer_all",
    full_name="gogoproto.sizer_all",
    index=22,
    number=63020,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
GOPROTO_ENUM_STRINGER_ALL_FIELD_NUMBER = 63021
goproto_enum_stringer_all = _descriptor.FieldDescriptor(
    name="goproto_enum_stringer_all",
    full_name="gogoproto.goproto_enum_stringer_all",
    index=23,
    number=63021,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
ENUM_STRINGER_ALL_FIELD_NUMBER = 63022
enum_stringer_all = _descriptor.FieldDescriptor(
    name="enum_stringer_all",
    full_name="gogoproto.enum_stringer_all",
    index=24,
    number=63022,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
UNSAFE_MARSHALER_ALL_FIELD_NUMBER = 63023
unsafe_marshaler_all = _descriptor.FieldDescriptor(
    name="unsafe_marshaler_all",
    full_name="gogoproto.unsafe_marshaler_all",
    index=25,
    number=63023,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
UNSAFE_UNMARSHALER_ALL_FIELD_NUMBER = 63024
unsafe_unmarshaler_all = _descriptor.FieldDescriptor(
    name="unsafe_unmarshaler_all",
    full_name="gogoproto.unsafe_unmarshaler_all",
    index=26,
    number=63024,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
GOPROTO_EXTENSIONS_MAP_ALL_FIELD_NUMBER = 63025
goproto_extensions_map_all = _descriptor.FieldDescriptor(
    name="goproto_extensions_map_all",
    full_name="gogoproto.goproto_extensions_map_all",
    index=27,
    number=63025,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
GOPROTO_UNRECOGNIZED_ALL_FIELD_NUMBER = 63026
goproto_unrecognized_all = _descriptor.FieldDescriptor(
    name="goproto_unrecognized_all",
    full_name="gogoproto.goproto_unrecognized_all",
    index=28,
    number=63026,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
GOGOPROTO_IMPORT_FIELD_NUMBER = 63027
gogoproto_import = _descriptor.FieldDescriptor(
    name="gogoproto_import",
    full_name="gogoproto.gogoproto_import",
    index=29,
    number=63027,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
PROTOSIZER_ALL_FIELD_NUMBER = 63028
protosizer_all = _descriptor.FieldDescriptor(
    name="protosizer_all",
    full_name="gogoproto.protosizer_all",
    index=30,
    number=63028,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
COMPARE_ALL_FIELD_NUMBER = 63029
compare_all = _descriptor.FieldDescriptor(
    name="compare_all",
    full_name="gogoproto.compare_all",
    index=31,
    number=63029,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
TYPEDECL_ALL_FIELD_NUMBER = 63030
typedecl_all = _descriptor.FieldDescriptor(
    name="typedecl_all",
    full_name="gogoproto.typedecl_all",
    index=32,
    number=63030,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
ENUMDECL_ALL_FIELD_NUMBER = 63031
enumdecl_all = _descriptor.FieldDescriptor(
    name="enumdecl_all",
    full_name="gogoproto.enumdecl_all",
    index=33,
    number=63031,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
GOPROTO_REGISTRATION_FIELD_NUMBER = 63032
goproto_registration = _descriptor.FieldDescriptor(
    name="goproto_registration",
    full_name="gogoproto.goproto_registration",
    index=34,
    number=63032,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
MESSAGENAME_ALL_FIELD_NUMBER = 63033
messagename_all = _descriptor.FieldDescriptor(
    name="messagename_all",
    full_name="gogoproto.messagename_all",
    index=35,
    number=63033,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
GOPROTO_SIZECACHE_ALL_FIELD_NUMBER = 63034
goproto_sizecache_all = _descriptor.FieldDescriptor(
    name="goproto_sizecache_all",
    full_name="gogoproto.goproto_sizecache_all",
    index=36,
    number=63034,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
GOPROTO_UNKEYED_ALL_FIELD_NUMBER = 63035
goproto_unkeyed_all = _descriptor.FieldDescriptor(
    name="goproto_unkeyed_all",
    full_name="gogoproto.goproto_unkeyed_all",
    index=37,
    number=63035,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
GOPROTO_GETTERS_FIELD_NUMBER = 64001
goproto_getters = _descriptor.FieldDescriptor(
    name="goproto_getters",
    full_name="gogoproto.goproto_getters",
    index=38,
    number=64001,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
GOPROTO_STRINGER_FIELD_NUMBER = 64003
goproto_stringer = _descriptor.FieldDescriptor(
    name="goproto_stringer",
    full_name="gogoproto.goproto_stringer",
    index=39,
    number=64003,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
VERBOSE_EQUAL_FIELD_NUMBER = 64004
verbose_equal = _descriptor.FieldDescriptor(
    name="verbose_equal",
    full_name="gogoproto.verbose_equal",
    index=40,
    number=64004,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
FACE_FIELD_NUMBER = 64005
face = _descriptor.FieldDescriptor(
    name="face",
    full_name="gogoproto.face",
    index=41,
    number=64005,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
GOSTRING_FIELD_NUMBER = 64006
gostring = _descriptor.FieldDescriptor(
    name="gostring",
    full_name="gogoproto.gostring",
    index=42,
    number=64006,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
POPULATE_FIELD_NUMBER = 64007
populate = _descriptor.FieldDescriptor(
    name="populate",
    full_name="gogoproto.populate",
    index=43,
    number=64007,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
STRINGER_FIELD_NUMBER = 67008
stringer = _descriptor.FieldDescriptor(
    name="stringer",
    full_name="gogoproto.stringer",
    index=44,
    number=67008,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
ONLYONE_FIELD_NUMBER = 64009
onlyone = _descriptor.FieldDescriptor(
    name="onlyone",
    full_name="gogoproto.onlyone",
    index=45,
    number=64009,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
EQUAL_FIELD_NUMBER = 64013
equal = _descriptor.FieldDescriptor(
    name="equal",
    full_name="gogoproto.equal",
    index=46,
    number=64013,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
DESCRIPTION_FIELD_NUMBER = 64014
description = _descriptor.FieldDescriptor(
    name="description",
    full_name="gogoproto.description",
    index=47,
    number=64014,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
TESTGEN_FIELD_NUMBER = 64015
testgen = _descriptor.FieldDescriptor(
    name="testgen",
    full_name="gogoproto.testgen",
    index=48,
    number=64015,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
BENCHGEN_FIELD_NUMBER = 64016
benchgen = _descriptor.FieldDescriptor(
    name="benchgen",
    full_name="gogoproto.benchgen",
    index=49,
    number=64016,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
MARSHALER_FIELD_NUMBER = 64017
marshaler = _descriptor.FieldDescriptor(
    name="marshaler",
    full_name="gogoproto.marshaler",
    index=50,
    number=64017,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
UNMARSHALER_FIELD_NUMBER = 64018
unmarshaler = _descriptor.FieldDescriptor(
    name="unmarshaler",
    full_name="gogoproto.unmarshaler",
    index=51,
    number=64018,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
STABLE_MARSHALER_FIELD_NUMBER = 64019
stable_marshaler = _descriptor.FieldDescriptor(
    name="stable_marshaler",
    full_name="gogoproto.stable_marshaler",
    index=52,
    number=64019,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
SIZER_FIELD_NUMBER = 64020
sizer = _descriptor.FieldDescriptor(
    name="sizer",
    full_name="gogoproto.sizer",
    index=53,
    number=64020,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
UNSAFE_MARSHALER_FIELD_NUMBER = 64023
unsafe_marshaler = _descriptor.FieldDescriptor(
    name="unsafe_marshaler",
    full_name="gogoproto.unsafe_marshaler",
    index=54,
    number=64023,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
UNSAFE_UNMARSHALER_FIELD_NUMBER = 64024
unsafe_unmarshaler = _descriptor.FieldDescriptor(
    name="unsafe_unmarshaler",
    full_name="gogoproto.unsafe_unmarshaler",
    index=55,
    number=64024,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
GOPROTO_EXTENSIONS_MAP_FIELD_NUMBER = 64025
goproto_extensions_map = _descriptor.FieldDescriptor(
    name="goproto_extensions_map",
    full_name="gogoproto.goproto_extensions_map",
    index=56,
    number=64025,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
GOPROTO_UNRECOGNIZED_FIELD_NUMBER = 64026
goproto_unrecognized = _descriptor.FieldDescriptor(
    name="goproto_unrecognized",
    full_name="gogoproto.goproto_unrecognized",
    index=57,
    number=64026,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
PROTOSIZER_FIELD_NUMBER = 64028
protosizer = _descriptor.FieldDescriptor(
    name="protosizer",
    full_name="gogoproto.protosizer",
    index=58,
    number=64028,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
COMPARE_FIELD_NUMBER = 64029
compare = _descriptor.FieldDescriptor(
    name="compare",
    full_name="gogoproto.compare",
    index=59,
    number=64029,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
TYPEDECL_FIELD_NUMBER = 64030
typedecl = _descriptor.FieldDescriptor(
    name="typedecl",
    full_name="gogoproto.typedecl",
    index=60,
    number=64030,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
MESSAGENAME_FIELD_NUMBER = 64033
messagename = _descriptor.FieldDescriptor(
    name="messagename",
    full_name="gogoproto.messagename",
    index=61,
    number=64033,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
GOPROTO_SIZECACHE_FIELD_NUMBER = 64034
goproto_sizecache = _descriptor.FieldDescriptor(
    name="goproto_sizecache",
    full_name="gogoproto.goproto_sizecache",
    index=62,
    number=64034,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
GOPROTO_UNKEYED_FIELD_NUMBER = 64035
goproto_unkeyed = _descriptor.FieldDescriptor(
    name="goproto_unkeyed",
    full_name="gogoproto.goproto_unkeyed",
    index=63,
    number=64035,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
NULLABLE_FIELD_NUMBER = 65001
nullable = _descriptor.FieldDescriptor(
    name="nullable",
    full_name="gogoproto.nullable",
    index=64,
    number=65001,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
EMBED_FIELD_NUMBER = 65002
embed = _descriptor.FieldDescriptor(
    name="embed",
    full_name="gogoproto.embed",
    index=65,
    number=65002,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
CUSTOMTYPE_FIELD_NUMBER = 65003
customtype = _descriptor.FieldDescriptor(
    name="customtype",
    full_name="gogoproto.customtype",
    index=66,
    number=65003,
    type=9,
    cpp_type=9,
    label=1,
    has_default_value=False,
    default_value=b"".decode("utf-8"),
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
CUSTOMNAME_FIELD_NUMBER = 65004
customname = _descriptor.FieldDescriptor(
    name="customname",
    full_name="gogoproto.customname",
    index=67,
    number=65004,
    type=9,
    cpp_type=9,
    label=1,
    has_default_value=False,
    default_value=b"".decode("utf-8"),
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
JSONTAG_FIELD_NUMBER = 65005
jsontag = _descriptor.FieldDescriptor(
    name="jsontag",
    full_name="gogoproto.jsontag",
    index=68,
    number=65005,
    type=9,
    cpp_type=9,
    label=1,
    has_default_value=False,
    default_value=b"".decode("utf-8"),
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
MORETAGS_FIELD_NUMBER = 65006
moretags = _descriptor.FieldDescriptor(
    name="moretags",
    full_name="gogoproto.moretags",
    index=69,
    number=65006,
    type=9,
    cpp_type=9,
    label=1,
    has_default_value=False,
    default_value=b"".decode("utf-8"),
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
CASTTYPE_FIELD_NUMBER = 65007
casttype = _descriptor.FieldDescriptor(
    name="casttype",
    full_name="gogoproto.casttype",
    index=70,
    number=65007,
    type=9,
    cpp_type=9,
    label=1,
    has_default_value=False,
    default_value=b"".decode("utf-8"),
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
CASTKEY_FIELD_NUMBER = 65008
castkey = _descriptor.FieldDescriptor(
    name="castkey",
    full_name="gogoproto.castkey",
    index=71,
    number=65008,
    type=9,
    cpp_type=9,
    label=1,
    has_default_value=False,
    default_value=b"".decode("utf-8"),
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
CASTVALUE_FIELD_NUMBER = 65009
castvalue = _descriptor.FieldDescriptor(
    name="castvalue",
    full_name="gogoproto.castvalue",
    index=72,
    number=65009,
    type=9,
    cpp_type=9,
    label=1,
    has_default_value=False,
    default_value=b"".decode("utf-8"),
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
STDTIME_FIELD_NUMBER = 65010
stdtime = _descriptor.FieldDescriptor(
    name="stdtime",
    full_name="gogoproto.stdtime",
    index=73,
    number=65010,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
STDDURATION_FIELD_NUMBER = 65011
stdduration = _descriptor.FieldDescriptor(
    name="stdduration",
    full_name="gogoproto.stdduration",
    index=74,
    number=65011,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)
WKTPOINTER_FIELD_NUMBER = 65012
wktpointer = _descriptor.FieldDescriptor(
    name="wktpointer",
    full_name="gogoproto.wktpointer",
    index=75,
    number=65012,
    type=8,
    cpp_type=7,
    label=1,
    has_default_value=False,
    default_value=False,
    message_type=None,
    enum_type=None,
    containing_type=None,
    is_extension=True,
    extension_scope=None,
    serialized_options=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
)

DESCRIPTOR.extensions_by_name["goproto_enum_prefix"] = goproto_enum_prefix
DESCRIPTOR.extensions_by_name["goproto_enum_stringer"] = goproto_enum_stringer
DESCRIPTOR.extensions_by_name["enum_stringer"] = enum_stringer
DESCRIPTOR.extensions_by_name["enum_customname"] = enum_customname
DESCRIPTOR.extensions_by_name["enumdecl"] = enumdecl
DESCRIPTOR.extensions_by_name["enumvalue_customname"] = enumvalue_customname
DESCRIPTOR.extensions_by_name["goproto_getters_all"] = goproto_getters_all
DESCRIPTOR.extensions_by_name["goproto_enum_prefix_all"] = goproto_enum_prefix_all
DESCRIPTOR.extensions_by_name["goproto_stringer_all"] = goproto_stringer_all
DESCRIPTOR.extensions_by_name["verbose_equal_all"] = verbose_equal_all
DESCRIPTOR.extensions_by_name["face_all"] = face_all
DESCRIPTOR.extensions_by_name["gostring_all"] = gostring_all
DESCRIPTOR.extensions_by_name["populate_all"] = populate_all
DESCRIPTOR.extensions_by_name["stringer_all"] = stringer_all
DESCRIPTOR.extensions_by_name["onlyone_all"] = onlyone_all
DESCRIPTOR.extensions_by_name["equal_all"] = equal_all
DESCRIPTOR.extensions_by_name["description_all"] = description_all
DESCRIPTOR.extensions_by_name["testgen_all"] = testgen_all
DESCRIPTOR.extensions_by_name["benchgen_all"] = benchgen_all
DESCRIPTOR.extensions_by_name["marshaler_all"] = marshaler_all
DESCRIPTOR.extensions_by_name["unmarshaler_all"] = unmarshaler_all
DESCRIPTOR.extensions_by_name["stable_marshaler_all"] = stable_marshaler_all
DESCRIPTOR.extensions_by_name["sizer_all"] = sizer_all
DESCRIPTOR.extensions_by_name["goproto_enum_stringer_all"] = goproto_enum_stringer_all
DESCRIPTOR.extensions_by_name["enum_stringer_all"] = enum_stringer_all
DESCRIPTOR.extensions_by_name["unsafe_marshaler_all"] = unsafe_marshaler_all
DESCRIPTOR.extensions_by_name["unsafe_unmarshaler_all"] = unsafe_unmarshaler_all
DESCRIPTOR.extensions_by_name["goproto_extensions_map_all"] = goproto_extensions_map_all
DESCRIPTOR.extensions_by_name["goproto_unrecognized_all"] = goproto_unrecognized_all
DESCRIPTOR.extensions_by_name["gogoproto_import"] = gogoproto_import
DESCRIPTOR.extensions_by_name["protosizer_all"] = protosizer_all
DESCRIPTOR.extensions_by_name["compare_all"] = compare_all
DESCRIPTOR.extensions_by_name["typedecl_all"] = typedecl_all
DESCRIPTOR.extensions_by_name["enumdecl_all"] = enumdecl_all
DESCRIPTOR.extensions_by_name["goproto_registration"] = goproto_registration
DESCRIPTOR.extensions_by_name["messagename_all"] = messagename_all
DESCRIPTOR.extensions_by_name["goproto_sizecache_all"] = goproto_sizecache_all
DESCRIPTOR.extensions_by_name["goproto_unkeyed_all"] = goproto_unkeyed_all
DESCRIPTOR.extensions_by_name["goproto_getters"] = goproto_getters
DESCRIPTOR.extensions_by_name["goproto_stringer"] = goproto_stringer
DESCRIPTOR.extensions_by_name["verbose_equal"] = verbose_equal
DESCRIPTOR.extensions_by_name["face"] = face
DESCRIPTOR.extensions_by_name["gostring"] = gostring
DESCRIPTOR.extensions_by_name["populate"] = populate
DESCRIPTOR.extensions_by_name["stringer"] = stringer
DESCRIPTOR.extensions_by_name["onlyone"] = onlyone
DESCRIPTOR.extensions_by_name["equal"] = equal
DESCRIPTOR.extensions_by_name["description"] = description
DESCRIPTOR.extensions_by_name["testgen"] = testgen
DESCRIPTOR.extensions_by_name["benchgen"] = benchgen
DESCRIPTOR.extensions_by_name["marshaler"] = marshaler
DESCRIPTOR.extensions_by_name["unmarshaler"] = unmarshaler
DESCRIPTOR.extensions_by_name["stable_marshaler"] = stable_marshaler
DESCRIPTOR.extensions_by_name["sizer"] = sizer
DESCRIPTOR.extensions_by_name["unsafe_marshaler"] = unsafe_marshaler
DESCRIPTOR.extensions_by_name["unsafe_unmarshaler"] = unsafe_unmarshaler
DESCRIPTOR.extensions_by_name["goproto_extensions_map"] = goproto_extensions_map
DESCRIPTOR.extensions_by_name["goproto_unrecognized"] = goproto_unrecognized
DESCRIPTOR.extensions_by_name["protosizer"] = protosizer
DESCRIPTOR.extensions_by_name["compare"] = compare
DESCRIPTOR.extensions_by_name["typedecl"] = typedecl
DESCRIPTOR.extensions_by_name["messagename"] = messagename
DESCRIPTOR.extensions_by_name["goproto_sizecache"] = goproto_sizecache
DESCRIPTOR.extensions_by_name["goproto_unkeyed"] = goproto_unkeyed
DESCRIPTOR.extensions_by_name["nullable"] = nullable
DESCRIPTOR.extensions_by_name["embed"] = embed
DESCRIPTOR.extensions_by_name["customtype"] = customtype
DESCRIPTOR.extensions_by_name["customname"] = customname
DESCRIPTOR.extensions_by_name["jsontag"] = jsontag
DESCRIPTOR.extensions_by_name["moretags"] = moretags
DESCRIPTOR.extensions_by_name["casttype"] = casttype
DESCRIPTOR.extensions_by_name["castkey"] = castkey
DESCRIPTOR.extensions_by_name["castvalue"] = castvalue
DESCRIPTOR.extensions_by_name["stdtime"] = stdtime
DESCRIPTOR.extensions_by_name["stdduration"] = stdduration
DESCRIPTOR.extensions_by_name["wktpointer"] = wktpointer
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

google_dot_protobuf_dot_descriptor__pb2.EnumOptions.RegisterExtension(
    goproto_enum_prefix
)
google_dot_protobuf_dot_descriptor__pb2.EnumOptions.RegisterExtension(
    goproto_enum_stringer
)
google_dot_protobuf_dot_descriptor__pb2.EnumOptions.RegisterExtension(enum_stringer)
google_dot_protobuf_dot_descriptor__pb2.EnumOptions.RegisterExtension(enum_customname)
google_dot_protobuf_dot_descriptor__pb2.EnumOptions.RegisterExtension(enumdecl)
google_dot_protobuf_dot_descriptor__pb2.EnumValueOptions.RegisterExtension(
    enumvalue_customname
)
google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(
    goproto_getters_all
)
google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(
    goproto_enum_prefix_all
)
google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(
    goproto_stringer_all
)
google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(verbose_equal_all)
google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(face_all)
google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(gostring_all)
google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(populate_all)
google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(stringer_all)
google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(onlyone_all)
google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(equal_all)
google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(description_all)
google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(testgen_all)
google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(benchgen_all)
google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(marshaler_all)
google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(unmarshaler_all)
google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(
    stable_marshaler_all
)
google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(sizer_all)
google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(
    goproto_enum_stringer_all
)
google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(enum_stringer_all)
google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(
    unsafe_marshaler_all
)
google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(
    unsafe_unmarshaler_all
)
google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(
    goproto_extensions_map_all
)
google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(
    goproto_unrecognized_all
)
google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(gogoproto_import)
google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(protosizer_all)
google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(compare_all)
google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(typedecl_all)
google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(enumdecl_all)
google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(
    goproto_registration
)
google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(messagename_all)
google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(
    goproto_sizecache_all
)
google_dot_protobuf_dot_descriptor__pb2.FileOptions.RegisterExtension(
    goproto_unkeyed_all
)
google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(
    goproto_getters
)
google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(
    goproto_stringer
)
google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(verbose_equal)
google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(face)
google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(gostring)
google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(populate)
google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(stringer)
google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(onlyone)
google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(equal)
google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(description)
google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(testgen)
google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(benchgen)
google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(marshaler)
google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(unmarshaler)
google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(
    stable_marshaler
)
google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(sizer)
google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(
    unsafe_marshaler
)
google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(
    unsafe_unmarshaler
)
google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(
    goproto_extensions_map
)
google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(
    goproto_unrecognized
)
google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(protosizer)
google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(compare)
google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(typedecl)
google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(messagename)
google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(
    goproto_sizecache
)
google_dot_protobuf_dot_descriptor__pb2.MessageOptions.RegisterExtension(
    goproto_unkeyed
)
google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(nullable)
google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(embed)
google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(customtype)
google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(customname)
google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(jsontag)
google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(moretags)
google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(casttype)
google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(castkey)
google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(castvalue)
google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(stdtime)
google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(stdduration)
google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(wktpointer)

DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2021-2022 Valory AG
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""This module contains the behaviours for the 'liquidity_rebalancing_abci' skill."""
from typing import Set, Type

from packages.valory.skills.abstract_round_abci.behaviours import (
    AbstractRoundBehaviour,
    BaseBehaviour,
)
from packages.valory.skills.liquidity_provision_abci.composition import (
    LiquidityProvisionAbciApp,
)
from packages.valory.skills.liquidity_rebalancing_abci.behaviours import (
    LiquidityRebalancingConsensusBehaviour,
)
from packages.valory.skills.registration_abci.behaviours import (
    AgentRegistrationRoundBehaviour,
    RegistrationStartupBehaviour,
)
from packages.valory.skills.reset_pause_abci.behaviours import (
    ResetPauseABCIConsensusBehaviour,
)
from packages.valory.skills.safe_deployment_abci.behaviours import (
    SafeDeploymentRoundBehaviour,
)
from packages.valory.skills.transaction_settlement_abci.behaviours import (
    TransactionSettlementRoundBehaviour,
)


class LiquidityProvisionConsensusBehaviour(AbstractRoundBehaviour):
    """This behaviour manages the consensus stages for the liquidity rebalancing."""

    initial_behaviour_cls = RegistrationStartupBehaviour
    abci_app_cls = LiquidityProvisionAbciApp  # type: ignore
    behaviours: Set[Type[BaseBehaviour]] = {
        *AgentRegistrationRoundBehaviour.behaviours,
        *SafeDeploymentRoundBehaviour.behaviours,
        *TransactionSettlementRoundBehaviour.behaviours,
        *ResetPauseABCIConsensusBehaviour.behaviours,
        *LiquidityRebalancingConsensusBehaviour.behaviours,
    }



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2021 Valory AG
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""This module contains the model 'state' for the 'counter_client' skill."""
from typing import Any, Dict

from aea.skills.base import Model


class Params(Model):
    """Parameters."""

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        """Initialize the parameters object."""
        super().__init__(*args, **kwargs)
        self.tendermint_url = kwargs.get("tendermint_url")


class State(Model):
    """Keep the current state."""

    def __init__(self, *args: Any, **kwargs: Any) -> None:
        """Initialize the state."""
        super().__init__(*args, **kwargs)
        self.count = 0

        # mapping from dialogue reference nonce to handler callback name
        self.request_to_handler: Dict[str, str] = {}



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2021-2022 Valory AG
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""This module contains the transaction payloads for common apps."""
from enum import Enum
from typing import Any, Dict, Optional

from packages.valory.skills.abstract_round_abci.base import BaseTxPayload


class TransactionType(Enum):
    """Enumeration of transaction types."""

    OBSERVATION = "observation"
    ESTIMATE = "estimate"
    TX_HASH = "tx_hash"

    def __str__(self) -> str:
        """Get the string value of the transaction type."""
        return self.value


class TransactionHashPayload(BaseTxPayload):
    """Represent a transaction payload of type 'tx_hash'."""

    transaction_type = TransactionType.TX_HASH

    def __init__(
        self, sender: str, tx_hash: Optional[str] = None, **kwargs: Any
    ) -> None:
        """Initialize an 'tx_hash' transaction payload.

        :param sender: the sender (Ethereum) address
        :param tx_hash: the tx_hash
        :param kwargs: the keyword arguments
        """
        super().__init__(sender, **kwargs)
        self._tx_hash = tx_hash

    @property
    def tx_hash(self) -> Optional[str]:
        """Get the tx_hash."""
        return self._tx_hash

    @property
    def data(self) -> Dict:
        """Get the data."""
        return dict(tx_hash=self.tx_hash) if self.tx_hash is not None else {}


class ObservationPayload(BaseTxPayload):
    """Represent a transaction payload of type 'observation'."""

    transaction_type = TransactionType.OBSERVATION

    def __init__(self, sender: str, observation: float, **kwargs: Any) -> None:
        """Initialize an 'observation' transaction payload.

        :param sender: the sender (Ethereum) address
        :param observation: the observation
        :param kwargs: the keyword arguments
        """
        super().__init__(sender, **kwargs)
        self._observation = observation

    @property
    def observation(self) -> float:
        """Get the observation."""
        return self._observation

    @property
    def data(self) -> Dict:
        """Get the data."""
        return dict(observation=self.observation)


class EstimatePayload(BaseTxPayload):
    """Represent a transaction payload of type 'estimate'."""

    transaction_type = TransactionType.ESTIMATE

    def __init__(self, sender: str, estimate: float, **kwargs: Any) -> None:
        """Initialize an 'estimate' transaction payload.

        :param sender: the sender (Ethereum) address
        :param estimate: the estimate
        :param kwargs: the keyword arguments
        """
        super().__init__(sender, **kwargs)
        self._estimate = estimate

    @property
    def estimate(self) -> float:
        """Get the estimate."""
        return self._estimate

    @property
    def data(self) -> Dict:
        """Get the data."""
        return dict(estimate=self.estimate)



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2021-2022 Valory AG
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""This module contains the data classes for the oracle deployment ABCI application."""

from typing import Generator, Optional, Set, Type, cast

from aea_ledger_ethereum import EthereumApi

from packages.valory.contracts.offchain_aggregator.contract import (
    OffchainAggregatorContract,
)
from packages.valory.protocols.contract_api import ContractApiMessage
from packages.valory.skills.abstract_round_abci.behaviour_utils import BaseBehaviour
from packages.valory.skills.abstract_round_abci.behaviours import AbstractRoundBehaviour
from packages.valory.skills.abstract_round_abci.common import (
    RandomnessBehaviour,
    SelectKeeperBehaviour,
)
from packages.valory.skills.oracle_deployment_abci.models import Params
from packages.valory.skills.oracle_deployment_abci.payloads import (
    DeployOraclePayload,
    RandomnessPayload,
    SelectKeeperPayload,
    ValidateOraclePayload,
)
from packages.valory.skills.oracle_deployment_abci.rounds import (
    DeployOracleRound,
    OracleDeploymentAbciApp,
    RandomnessOracleRound,
    SelectKeeperOracleRound,
    SynchronizedData,
    ValidateOracleRound,
)


class OracleDeploymentBaseBehaviour(BaseBehaviour):
    """Base behaviour for the common apps' skill."""

    @property
    def synchronized_data(self) -> SynchronizedData:
        """Return the synchronized data."""
        return cast(SynchronizedData, super().synchronized_data)

    @property
    def params(self) -> Params:
        """Return the synchronized data."""
        return cast(Params, super().params)


class RandomnessOracleBehaviour(RandomnessBehaviour):
    """Retrieve randomness for oracle deployment."""

    behaviour_id = "retrieve_randomness_oracle"
    matching_round = RandomnessOracleRound
    payload_class = RandomnessPayload


class SelectKeeperOracleBehaviour(SelectKeeperBehaviour):
    """Select the keeper agent."""

    behaviour_id = "select_keeper_oracle"
    matching_round = SelectKeeperOracleRound
    payload_class = SelectKeeperPayload


class DeployOracleBehaviour(OracleDeploymentBaseBehaviour):
    """Deploy oracle."""

    behaviour_id = "deploy_oracle"
    matching_round = DeployOracleRound

    def async_act(self) -> Generator:
        """
        Do the action.

        Steps:
        - If the agent is the designated deployer, then prepare the deployment
          transaction and send it.
        - Otherwise, wait until the next round.
        - If a timeout is hit, set exit A event, otherwise set done event.
        """
        if (
            self.context.agent_address
            != self.synchronized_data.most_voted_keeper_address
        ):
            yield from self._not_deployer_act()
        else:
            yield from self._deployer_act()

    def _not_deployer_act(self) -> Generator:
        """Do the non-deployer action."""
        with self.context.benchmark_tool.measure(self.behaviour_id).consensus():
            yield from self.wait_until_round_end()
            self.set_done()

    def _deployer_act(self) -> Generator:
        """Do the deployer action."""

        with self.context.benchmark_tool.measure(self.behaviour_id).local():
            self.context.logger.info(
                "I am the designated sender, deploying the oracle contract..."
            )
            contract_address = yield from self._send_deploy_transaction()
            if contract_address is None:
                # The oracle_deployment_abci app should only be used in staging.
                # If the oracle contract deployment fails we abort. Alternatively,
                # we could send a None payload and then transition into an appropriate
                # round to handle the deployment failure.
                raise RuntimeError("Oracle deployment failed!")  # pragma: nocover
            payload = DeployOraclePayload(self.context.agent_address, contract_address)

        with self.context.benchmark_tool.measure(self.behaviour_id).consensus():
            self.context.logger.info(f"Oracle contract address: {contract_address}")
            yield from self.send_a2a_transaction(payload)
            yield from self.wait_until_round_end()

        self.set_done()

    def _send_deploy_transaction(self) -> Generator[None, None, Optional[str]]:
        min_answer = self.params.oracle_params["min_answer"]
        max_answer = self.params.oracle_params["max_answer"]
        decimals = self.params.oracle_params["decimals"]
        description = self.params.oracle_params["description"]
        contract_api_response = yield from self.get_contract_api_response(
            performative=ContractApiMessage.Performative.GET_DEPLOY_TRANSACTION,  # type: ignore
            contract_address=None,
            contract_id=str(OffchainAggregatorContract.contract_id),
            contract_callable="get_deploy_transaction",
            deployer_address=self.context.agent_address,
            _minAnswer=min_answer,
            _maxAnswer=max_answer,
            _decimals=decimals,
            _description=description,
            _transmitters=[self.synchronized_data.safe_contract_address],
        )
        if (
            contract_api_response.performative
            != ContractApiMessage.Performative.RAW_TRANSACTION
        ):  # pragma: nocover
            self.context.logger.warning("get_deploy_transaction unsuccessful!")
            return None
        tx_digest, _ = yield from self.send_raw_transaction(
            contract_api_response.raw_transaction
        )
        if tx_digest is None:  # pragma: nocover
            self.context.logger.warning("send_raw_transaction unsuccessful!")
            return None
        tx_receipt = yield from self.get_transaction_receipt(tx_digest)
        if tx_receipt is None:  # pragma: nocover
            self.context.logger.warning("get_transaction_receipt unsuccessful!")
            return None
        contract_address = EthereumApi.get_contract_address(tx_receipt)
        self.context.logger.info(f"Deployment tx digest: {tx_digest}")
        return contract_address


class ValidateOracleBehaviour(OracleDeploymentBaseBehaviour):
    """Validate oracle."""

    behaviour_id = "validate_oracle"
    matching_round = ValidateOracleRound

    def async_act(self) -> Generator:
        """
        Do the action.

        Steps:
        - Validate that the contract address provided by the keeper points to a
          valid contract.
        - Send the transaction with the validation result and wait for it to be
          mined.
        - Wait until ABCI application transitions to the next round.
        - Go to the next behaviour (set done event).
        """

        with self.context.benchmark_tool.measure(self.behaviour_id).local():
            is_correct = yield from self.has_correct_contract_been_deployed()
            payload = ValidateOraclePayload(self.context.agent_address, is_correct)

        with self.context.benchmark_tool.measure(self.behaviour_id).consensus():
            yield from self.send_a2a_transaction(payload)
            yield from self.wait_until_round_end()

        self.set_done()

    def has_correct_contract_been_deployed(self) -> Generator[None, None, bool]:
        """Contract deployment verification."""
        contract_api_response = yield from self.get_contract_api_response(
            performative=ContractApiMessage.Performative.GET_STATE,  # type: ignore
            contract_address=self.synchronized_data.oracle_contract_address,
            contract_id=str(OffchainAggregatorContract.contract_id),
            contract_callable="verify_contract",
        )
        if (
            contract_api_response.performative != ContractApiMessage.Performative.STATE
        ):  # pragma: nocover
            self.context.logger.warning("verify_contract unsuccessful!")
            return False
        verified = cast(bool, contract_api_response.state.body["verified"])
        return verified


class OracleDeploymentRoundBehaviour(AbstractRoundBehaviour):
    """This behaviour manages the consensus stages for the oracle deployment."""

    initial_behaviour_cls = RandomnessOracleBehaviour
    abci_app_cls = OracleDeploymentAbciApp
    behaviours: Set[Type[BaseBehaviour]] = {  # type: ignore
        RandomnessOracleBehaviour,  # type: ignore
        SelectKeeperOracleBehaviour,  # type: ignore
        DeployOracleBehaviour,  # type: ignore
        ValidateOracleBehaviour,  # type: ignore
    }



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2021-2022 Valory AG
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Tests for valory/offchain_aggregator contract."""

import time
from pathlib import Path
from typing import Any, Dict, List, Tuple, cast

import pytest
from aea.crypto.registries import crypto_registry
from aea_ledger_ethereum import EthereumCrypto
from web3 import Web3

from packages.valory.contracts.offchain_aggregator.contract import (
    OffchainAggregatorContract,
)

from tests.conftest import ETHEREUM_KEY_PATH_1, ROOT_DIR
from tests.test_contracts.base import BaseGanacheContractTest


class BaseContractTest(BaseGanacheContractTest):
    """Base test case for OffchainAggregatorContract"""

    MIN_ANSWER: int = 10 ** 18
    MAX_ANSWER: int = 10 ** 24
    DECIMALS: int = 18
    DESCRIPTION: str = "BTC"
    NB_TRANSMITTERS: int = 1
    GAS: int = 10 ** 10
    DEFAULT_MAX_FEE_PER_GAS: int = 10 ** 10
    DEFAULT_MAX_PRIORITY_FEE_PER_GAS: int = 10 ** 10
    contract_directory = Path(
        ROOT_DIR, "packages", "valory", "contracts", "offchain_aggregator"
    )
    contract: OffchainAggregatorContract

    @classmethod
    def deployment_kwargs(cls) -> Dict[str, Any]:
        """Get deployment kwargs."""
        return dict(
            _minAnswer=cls.MIN_ANSWER,
            _maxAnswer=cls.MAX_ANSWER,
            _decimals=cls.DECIMALS,
            _description=cls.DESCRIPTION,
            _transmitters=cls.transmitters(),
            gas=cls.GAS,
        )

    @classmethod
    def transmitters(cls) -> List[str]:
        """Get the owners."""
        return [
            Web3.toChecksumAddress(t[0])
            for t in cls.key_pairs()[1 : cls.NB_TRANSMITTERS + 1]
        ]

    @classmethod
    def deployer(cls) -> Tuple[str, str]:
        """Get the key pair of the deployer."""
        # for simplicity, get the first owner
        return cls.key_pairs()[0]


class TestDeployTransaction(BaseContractTest):
    """Test."""

    def test_deployed(self) -> None:
        """Run tests."""
        result = self.contract.get_deploy_transaction(
            ledger_api=self.ledger_api,
            deployer_address=str(self.deployer_crypto.address),
            **self.deployment_kwargs()
        )
        assert type(result) == dict
        assert len(result) == 8
        data = result.pop("data")
        assert type(data) == str
        assert len(data) > 0 and data.startswith("0x")
        assert all(
            [
                key in result
                for key in [
                    "value",
                    "from",
                    "gas",
                    "maxFeePerGas",
                    "maxPriorityFeePerGas",
                    "nonce",
                ]
            ]
        ), "Error, found: {}".format(result)

    def test_non_implemented_methods(
        self,
    ) -> None:
        """Test not implemented methods."""
        with pytest.raises(NotImplementedError):
            self.contract.get_raw_transaction(self.ledger_api, "")

        with pytest.raises(NotImplementedError):
            self.contract.get_raw_message(self.ledger_api, "")

        with pytest.raises(NotImplementedError):
            self.contract.get_state(self.ledger_api, "")

    def test_get_transmit_data(self) -> None:
        """Run test_get_transmit_data test."""
        assert self.contract_address is not None
        result = self.contract.get_transmit_data(
            ledger_api=self.ledger_api,
            contract_address=self.contract_address,
            epoch_=1,
            round_=1,
            amount_=10 ** 19,
        )
        assert result["data"], "Contract did not return data."

    def test_get_latest_transmission_details(self) -> None:
        """Run get_latest_transmission_details test."""
        assert self.contract_address is not None
        result = self.contract.get_latest_transmission_details(
            ledger_api=self.ledger_api,
            contract_address=self.contract_address,
        )
        assert all(key in ["epoch_", "round_"] for key in result.keys())

    def test_verify(self) -> None:
        """Run verify test."""
        assert self.contract_address is not None
        result = self.contract.verify_contract(
            ledger_api=self.ledger_api,
            contract_address=self.contract_address,
        )
        assert result["verified"], "Contract not verified."

    def test_transmit_and_get_latest_round_data(self) -> None:
        """Run transmit test."""
        assert self.contract_address is not None
        epoch_ = 1
        round_ = 2
        amount_ = 10 ** 19
        result = self.contract.transmit(
            ledger_api=self.ledger_api,
            contract_address=self.contract_address,
            epoch_=epoch_,
            round_=round_,
            amount_=amount_,
            sender_address=self.transmitters()[0],
            gas=self.GAS,
        )
        assert result is not None, "Tx generation failed"
        assert len(result) == 8
        sender = crypto_registry.make(
            EthereumCrypto.identifier, private_key_path=ETHEREUM_KEY_PATH_1
        )
        assert sender.address == self.transmitters()[0]
        tx_signed = sender.sign_transaction(result)
        tx_hash = self.ledger_api.send_signed_transaction(tx_signed)
        assert tx_hash is not None, "Tx hash is none"
        time.sleep(1)
        tx_receipt = self.ledger_api.get_transaction_receipt(tx_hash)
        assert tx_receipt is not None, "Tx receipt is none"
        result_ = cast(
            List,
            self.contract.latest_round_data(
                ledger_api=self.ledger_api,
                contract_address=self.contract_address,
            ),
        )
        assert type(result_) == list, "Call failed"
        assert result_[0] == epoch_
        assert result_[1] == amount_
        assert result_[4] == epoch_



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2021-2022 Valory AG
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Test the tools.py module of the skill."""

import sys

import atheris  # type: ignore
import pytest

from packages.valory.skills.abstract_round_abci.common import random_selection
from packages.valory.skills.price_estimation_abci.behaviours import to_int


def test_random_selection_function() -> None:
    """Test `random_selection` function."""

    assert random_selection(["hello", "world", "!"], 0.1) == "hello"


@pytest.mark.parametrize("value", (-1, 2))
def test_random_selection_function_raises(value: int) -> None:
    """Test `random_selection` function."""
    with pytest.raises(ValueError):
        random_selection(["hello", "world", "!"], value)


def test_to_int_positive() -> None:
    """Test `to_int` function."""
    assert to_int(0.542, 5) == 54200
    assert to_int(0.542, 2) == 54
    assert to_int(542, 2) == 54200


@pytest.mark.skip
def test_fuzz_to_int() -> None:
    """Test fuzz to_int."""

    @atheris.instrument_func
    def fuzz_to_int(input_bytes: bytes) -> None:
        """Fuzz to_int."""
        fdp = atheris.FuzzedDataProvider(input_bytes)
        estimate = fdp.ConsumeFloat()
        decimals = fdp.ConsumeInt(4)
        to_int(estimate, decimals)

    atheris.instrument_all()
    atheris.Setup(sys.argv, fuzz_to_int)
    atheris.Fuzz()


@pytest.mark.skip
def test_fuzz_random_selection() -> None:
    """Test fuzz random_selection."""

    @atheris.instrument_func
    def fuzz_random_selection(input_bytes: bytes) -> None:
        """Fuzz random_selection."""
        fdp = atheris.FuzzedDataProvider(input_bytes)
        elements = [fdp.ConsumeString(12) for _ in range(4)]
        randomness = fdp.ConsumeFloat()
        if randomness > 1 or randomness < 0:
            return
        random_selection(elements, randomness)

    atheris.instrument_all()
    atheris.Setup(sys.argv, fuzz_random_selection)
    atheris.Fuzz()



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2021-2022 Valory AG
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Ganache Docker Image."""
import logging
import time
from typing import Dict, List, Optional

import requests
from aea.exceptions import enforce
from docker import DockerClient
from docker.models.containers import Container

from tests.helpers.docker.base import DockerImage


DEFAULT_GANACHE_ADDR = "http://127.0.0.1"
DEFAULT_GANACHE_PORT = 8545
DEFAULT_GANACHE_CHAIN_ID = 1337


class GanacheDockerImage(DockerImage):
    """Wrapper to Ganache Docker image."""

    def __init__(
        self,
        client: DockerClient,
        addr: str,
        port: int,
        config: Optional[Dict] = None,
        gas_limit: str = "0x9184e72a000",  # 10000000000000,
    ):
        """
        Initialize the Ganache Docker image.

        :param client: the Docker client.
        :param addr: the address.
        :param port: the port.
        :param config: optional configuration to command line.
        :param gas_limit: the gas limit for blocks.
        """
        super().__init__(client)
        self._addr = addr
        self._port = port
        self._config = config or {}
        self._gas_limit = gas_limit

    @property
    def tag(self) -> str:
        """Get the image tag."""
        return "trufflesuite/ganache:beta"

    def _make_ports(self) -> Dict:
        """Make ports dictionary for Docker."""
        return {f"{self._port}/tcp": ("0.0.0.0", self._port)}  # nosec

    def _build_command(self) -> List[str]:
        """Build command."""
        # cmd = ["--chain.hardfork=london"]  # noqa: E800
        cmd = ["--miner.blockGasLimit=" + str(self._gas_limit)]
        # cmd += ["--miner.callGasLimit=" + "0x1fffffffffffff"]  # noqa: E800
        accounts_balances = self._config.get("accounts_balances", [])
        for account, balance in accounts_balances:
            cmd += [f"--wallet.accounts='{account},{balance}'"]
        return cmd

    def create(self) -> Container:
        """Create the container."""
        cmd = self._build_command()
        container = self._client.containers.run(
            self.tag, command=cmd, detach=True, ports=self._make_ports()
        )
        return container

    def create_many(self, nb_containers: int) -> List[Container]:
        """Instantiate the image in many containers, parametrized."""
        raise NotImplementedError()

    def wait(self, max_attempts: int = 15, sleep_rate: float = 1.0) -> bool:
        """Wait until the image is up."""
        request = dict(jsonrpc=2.0, method="web3_clientVersion", params=[], id=1)
        for i in range(max_attempts):
            try:
                response = requests.post(f"{self._addr}:{self._port}", json=request)
                enforce(response.status_code == 200, "")
                return True
            except Exception:
                logging.error(
                    "Attempt %s failed. Retrying in %s seconds...", i, sleep_rate
                )
                time.sleep(sleep_rate)
        return False



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022 Valory AG
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Test `benchmarks` command"""

import json
import os
from contextlib import suppress
from pathlib import Path
from typing import Dict, List, Tuple

from autonomy.analyse.benchmark.aggregate import BlockTypes

from tests.test_autonomy.test_cli.base import BaseCliTest


NUMBER_OF_AGENTS = 4
NUMBER_OF_PERIODS = 3
NUMBER_OF_BEHAVIOURS = 3


def generate_benchmark_data(
    number_or_behaviours: int = NUMBER_OF_BEHAVIOURS,
) -> List[Dict]:
    """Generate dummy benchmark data."""

    return [
        {
            "behaviour": f"behaviour_{behaviour}",
            "data": {
                "local": 7.033348083496094e-05,
                "consensus": 3.771540880203247,
                "total": 3.771611213684082,
            },
        }
        for behaviour in range(number_or_behaviours)
    ]


class TestBenchmarks(BaseCliTest):
    """Test benchmarks tool."""

    cli_options: Tuple[str, ...] = ("analyse", "benchmarks")

    benchmarks_dir: Path
    output_file: Path

    @classmethod
    def setup(cls) -> None:
        """Setup class."""

        super().setup()

        cls.benchmarks_dir = cls.t / "benchmarks"
        cls.benchmarks_dir.mkdir()
        cls.output_file = cls.t / "benchmarks.html"

        for agent in range(NUMBER_OF_AGENTS):
            agent_dir = cls.benchmarks_dir / f"agent_{agent}"
            agent_dir.mkdir()
            for i in range(NUMBER_OF_PERIODS):
                period_file = agent_dir / f"{i}.json"
                period_file.write_text(json.dumps(generate_benchmark_data()))

        os.chdir(cls.t)

    def _run_test(self, block_type: str) -> None:
        """Test run."""

        with suppress(FileNotFoundError):
            os.remove(self.output_file)

        result = self.run_cli(
            (
                str(self.benchmarks_dir),
                "--output",
                str(self.output_file),
                f"--block-type={block_type}",
            ),
        )

        assert result.exit_code == 0
        assert self.output_file.exists()

        file_content = self.output_file.read_text()
        block_types = [BlockTypes.LOCAL, BlockTypes.CONSENSUS, BlockTypes.TOTAL]

        if block_type == BlockTypes.ALL:
            assert any([f"Block: {b}" in file_content for b in block_types])
        else:
            block_types.remove(block_type)
            assert f"Block: {block_type}" in file_content
            assert any([f"Block: {b}" not in file_content for b in block_types])

    def test_all_blocks(
        self,
    ) -> None:
        """Test with only local blocks."""

        self._run_test(BlockTypes.ALL)

    def test_only_local_blocks(
        self,
    ) -> None:
        """Test with all blocks."""

        self._run_test(BlockTypes.LOCAL)

    def test_only_consensus_blocks(
        self,
    ) -> None:
        """Test with only consensus blocks."""

        self._run_test(BlockTypes.CONSENSUS)

    def test_only_total_blocks(
        self,
    ) -> None:
        """Test with only total blocks."""

        self._run_test(BlockTypes.TOTAL)

    @classmethod
    def teardown(cls) -> None:
        """Teardown class."""
        os.chdir(cls.cwd)
        super().teardown()



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2021-2022 Valory AG
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""This module contains all the loading operations of the behaviours."""

import json
import os.path
from abc import ABC, abstractmethod
from typing import Callable, Dict, Optional

import joblib
import pandas as pd

from packages.valory.skills.abstract_round_abci.io.store import (
    CustomObjectType,
    NativelySupportedSingleObjectType,
    SupportedFiletype,
    SupportedObjectType,
    SupportedSingleObjectType,
)


CustomLoaderType = Optional[Callable[[str], CustomObjectType]]
SupportedLoaderType = Callable[[str], SupportedSingleObjectType]


class AbstractLoader(ABC):
    """An abstract `Loader` class."""

    @abstractmethod
    def load_single_file(self, path: str) -> SupportedSingleObjectType:
        """Load a single file."""

    def load(self, path: str, multiple: bool) -> SupportedObjectType:
        """Load one or more files.

        :param path: the path to the file to load. If multiple, then the path should be a folder with the files.
        :param multiple: whether multiple files are expected to be loaded. The path should be a folder with the files.
        :return: the loaded file.
        """
        if multiple:
            if not os.path.isdir(path):  # pragma: no cover
                raise ValueError(
                    f"Cannot load multiple files from `{path}`! "
                    f"Please make sure that the path is a folder containing the files."
                )

            objects = {}
            for filename in os.listdir(path):
                filename = os.fsdecode(filename)
                filepath = os.path.join(path, filename)
                objects[filename] = self.load_single_file(filepath)

            return objects

        return self.load_single_file(path)


class CSVLoader(AbstractLoader):
    """A csv files Loader."""

    def load_single_file(self, path: str) -> NativelySupportedSingleObjectType:
        """Read a pandas dataframe from a csv file.

        :param path: the path of the csv.
        :return: the pandas dataframe.
        """
        try:
            return pd.read_csv(path)
        except FileNotFoundError as e:  # pragma: no cover
            raise IOError(f"File {path} was not found!") from e
        except pd.errors.EmptyDataError as e:  # pragma: no cover
            raise IOError("The provided csv was empty!") from e


class ForecasterLoader(AbstractLoader):
    """A `pmdarima` forecaster loader."""

    def load_single_file(self, path: str) -> NativelySupportedSingleObjectType:
        """Load a `pmdarima` forecaster.

        :param path: path to store the forecaster.
        :return: a `pmdarima.pipeline.Pipeline`.
        """
        try:
            return joblib.load(path)
        except (NotADirectoryError, FileNotFoundError) as e:  # pragma: no cover
            raise IOError(f"Could not detect {path}!") from e


class JSONLoader(AbstractLoader):
    """A JSON file loader."""

    def load_single_file(self, path: str) -> NativelySupportedSingleObjectType:
        """Read a json file.

        :param path: the path to retrieve the json file from.
        :return: the deserialized json file's content.
        """
        try:
            with open(path, "r", encoding="utf-8") as f:
                return json.load(f)
        except OSError as e:  # pragma: no cover
            raise IOError(f"Path '{path}' could not be found!") from e

        except json.JSONDecodeError as e:  # pragma: no cover
            raise IOError(f"File '{path}' has an invalid JSON encoding!") from e

        except ValueError as e:  # pragma: no cover
            raise IOError(f"There is an encoding error in the '{path}' file!") from e


class Loader(AbstractLoader):
    """Class which loads files."""

    def __init__(
        self, filetype: Optional[SupportedFiletype], custom_loader: CustomLoaderType
    ):
        """Initialize a `Loader`."""
        self._filetype = filetype
        self._custom_loader = custom_loader
        self.__filetype_to_loader: Dict[SupportedFiletype, SupportedLoaderType] = {
            SupportedFiletype.JSON: JSONLoader().load_single_file,
            SupportedFiletype.PM_PIPELINE: ForecasterLoader().load_single_file,
            SupportedFiletype.CSV: CSVLoader().load_single_file,
        }

    def load_single_file(self, path: str) -> SupportedSingleObjectType:
        """Load a single file."""
        loader = self._get_single_loader_from_filetype()
        return loader(path)

    def _get_single_loader_from_filetype(self) -> SupportedLoaderType:
        """Get a file loader from a given filetype or keep a custom loader."""
        if self._filetype is not None:
            return self.__filetype_to_loader[self._filetype]

        if self._custom_loader is not None:  # pragma: no cover
            return self._custom_loader

        raise ValueError(  # pragma: no cover
            "Please provide either a supported filetype or a custom loader function."
        )
